searchData={"items":[{"type":"module","doc":"This module generates an Nginx configuration file using a templating system.\nAllows for template overrides in the current project.\nThe configuration is based on Horizon.Project and Horizon.Server.","title":"Horizon.NginxConfig","ref":"Horizon.NginxConfig.html"},{"type":"module","doc":"To customize your `nginx.conf`, copy the template from the Horizon package to your project:\n\n    $ mkdir -p priv/horizon/templates\n    $ cp deps/horizon/priv/templates/nginx/nginx.conf.eex priv/horizon/templates/nginx.conf.eex","title":"Customizing the Nginx configuration template - Horizon.NginxConfig","ref":"Horizon.NginxConfig.html#module-customizing-the-nginx-configuration-template"},{"type":"module","doc":"","title":"Running the generator from iex - Horizon.NginxConfig","ref":"Horizon.NginxConfig.html#module-running-the-generator-from-iex"},{"type":"module","doc":"```elixir\nuser = \"username\"\nhost = \"host-address\"\nremote_path = \"/usr/local/etc/nginx/nginx.conf\"\n\nprojects = [\n  %Horizon.Project{\n    name: \"project-name\",\n    server_names: [\"my.server.com\"],\n    http_only: true,\n    servers: [\n      %Horizon.Server{internal_ip: \"127.0.0.1\", port: 4000},\n      %Horizon.Server{internal_ip: \"192.168.100.100\", port: 4000}\n    ]\n  }\n]\n\nconfig_output = Horizon.NginxConfig.generate(projects)\nencoded_content = :base64.encode(config_output)\ncommand = \"echo #{encoded_content} | ssh #{user}@#{host} 'base64 -d | doas tee #{remote_path} > /dev/null && doas service nginx reload'\"\n{result, exit_code} = System.cmd(\"sh\", [\"-c\", command])\n```","title":"Examples - Horizon.NginxConfig","ref":"Horizon.NginxConfig.html#module-examples"},{"type":"function","doc":"Returns the path for the certificate","title":"Horizon.NginxConfig.cert_key_path/1","ref":"Horizon.NginxConfig.html#cert_key_path/1"},{"type":"function","doc":"iex> cert_key_path(%Horizon.Project{\n    ...>   certificate: :self,\n    ...>   cert_key_path: \"/path/to/cert_key.pem\"\n    ...> })\n    \"/path/to/cert_key.pem\"\n\n    iex> cert_key_path(%Horizon.Project{certificate: :letsencrypt, domain: \"example.com\"})\n    \"/user/local/etc/letsencrypt/live/example.com/privkey.pem\"","title":"Examples - Horizon.NginxConfig.cert_key_path/1","ref":"Horizon.NginxConfig.html#cert_key_path/1-examples"},{"type":"function","doc":"Returns the path for the certificate","title":"Horizon.NginxConfig.cert_path/1","ref":"Horizon.NginxConfig.html#cert_path/1"},{"type":"function","doc":"iex> cert_path(%Horizon.Project{certificate: :self, cert_path: \"/path/to/cert.pem\"})\n    \"/path/to/cert.pem\"\n\n    iex> cert_path(%Horizon.Project{certificate: :letsencrypt, domain: \"example.com\"})\n    \"/user/local/etc/letsencrypt/live/example.com/fullchain.pem\"","title":"Examples - Horizon.NginxConfig.cert_path/1","ref":"Horizon.NginxConfig.html#cert_path/1-examples"},{"type":"function","doc":"Generates an Nginx configuration file using a templating system.","title":"Horizon.NginxConfig.generate/1","ref":"Horizon.NginxConfig.html#generate/1"},{"type":"function","doc":"iex> project = %Horizon.Project{\n      name: \"example_project\",\n      server_names: [\"example.com\", \"www.example.com\"],\n      letsencrypt_live: \"mydomain.com\",\n      acme_challenge_path: \"/apps/challenge/mydomain.com\",\n      http_only: false,\n      servers: [\n        %Horizon.Server{internal_ip: \"10.0.0.1\", port: 4000},\n        %Horizon.Server{internal_ip: \"10.0.0.2\", port: 4001}\n      ]\n    }\n    iex> Horizon.NginxConfig.generate([project])","title":"Examples - Horizon.NginxConfig.generate/1","ref":"Horizon.NginxConfig.html#generate/1-examples"},{"type":"function","doc":"Sends the Nginx configuration to a remote host and reloads the Nginx service.","title":"Horizon.NginxConfig.send/4","ref":"Horizon.NginxConfig.html#send/4"},{"type":"function","doc":"iex> user = \"me\"\n    ...> host = \"myhost\"\n    ...> projects = [%Horizon.Project{name: \"my project\", ...}]\n    ...> NginxConfig.send(projects, user, host)\n    ...> NginxConfig.send(projects, user, host, nginxconf_path: \"/usr/nginx/nginx.conf\", action: :restart)","title":"Example - Horizon.NginxConfig.send/4","ref":"Horizon.NginxConfig.html#send/4-example"},{"type":"module","doc":"Horizon.Ops configuration.","title":"Horizon.Ops.BSD.Config","ref":"Horizon.Ops.BSD.Config.html"},{"type":"function","doc":"Merge default options with release options.","title":"Horizon.Ops.BSD.Config.merge_default_options/2","ref":"Horizon.Ops.BSD.Config.html#merge_default_options/2"},{"type":"function","doc":"iex> merge_default_options([], \"foo\")\n      [\n        path: \"/usr/local/foo\",\n        bin_path: \"bin\",\n        build_path: \"/usr/local/opt/foo/build\",\n        releases_path: \".releases\",\n        build_host_ssh: \"USER@HOSTUNKNOWN\",\n      ]","title":"Examples - Horizon.Ops.BSD.Config.merge_default_options/2","ref":"Horizon.Ops.BSD.Config.html#merge_default_options/2-examples"},{"type":"function","doc":"Merge default options into release.options and\nset release.path and release.version_path.","title":"Horizon.Ops.BSD.Config.merge_defaults/1","ref":"Horizon.Ops.BSD.Config.html#merge_defaults/1"},{"type":"function","doc":"iex> releases = [my_app: [path: \"/usr/apps/my_app\"]]\n    iex> merge_defaults(releases)\n    [\n      my_app: [\n        bin_path: \"bin\",\n        build_path: \"/usr/local/opt/my_app/build\",\n        releases_path: \".releases\",\n        build_host_ssh: \"USER@HOSTUNKNOWN\",\n        path: \"/usr/apps/my_app\"\n      ]\n    ]\n    iex> merge_defaults(my_app: [])\n    [\n      my_app: [\n        path: \"/usr/local/my_app\",\n        bin_path: \"bin\",\n        build_path: \"/usr/local/opt/my_app/build\",\n        releases_path: \".releases\",\n        build_host_ssh: \"USER@HOSTUNKNOWN\",\n      ]\n    ]","title":"Examples - Horizon.Ops.BSD.Config.merge_defaults/1","ref":"Horizon.Ops.BSD.Config.html#merge_defaults/1-examples"},{"type":"function","doc":"Merge default options into release.options and\nset release.path and release.version_path.","title":"Horizon.Ops.BSD.Config.merge_release/1","ref":"Horizon.Ops.BSD.Config.html#merge_release/1"},{"type":"module","doc":"The Horizon.Ops.BSD.Step module contains steps that are used to\nperform tasks during the release process.\n\n- `setup/1` - Run all the needed release steps.\n- `echo/1` - Echo the release name and options to the console.\n- `merge_defaults/1` - Merges the Horizon.Ops defaults into the release options.\n- `setup_rcd/1` - Create the rc.d script for the release.","title":"Horizon.Ops.BSD.Step","ref":"Horizon.Ops.BSD.Step.html"},{"type":"function","doc":"Echo the release name and options to the console.","title":"Horizon.Ops.BSD.Step.echo/1","ref":"Horizon.Ops.BSD.Step.html#echo/1"},{"type":"function","doc":"Merges the Horizon.Ops.BSD defaults into the release options.\n\nCalling this step will override Elixir's default value for\n`release.path` and `release.version_path`.","title":"Horizon.Ops.BSD.Step.merge_defaults/1","ref":"Horizon.Ops.BSD.Step.html#merge_defaults/1"},{"type":"function","doc":"Run the merge and rcd release steps.","title":"Horizon.Ops.BSD.Step.setup/1","ref":"Horizon.Ops.BSD.Step.html#setup/1"},{"type":"function","doc":"Create the rc.d script for the release.\nThis script is created `overlays/rc_d` and copied to `/usr/local/etc/rc.d/ `\nduring deploy.","title":"Horizon.Ops.BSD.Step.setup_rcd/1","ref":"Horizon.Ops.BSD.Step.html#setup_rcd/1"},{"type":"module","doc":"Horizon.Ops is a tool for building and deploying\nElixir applications to FreeBSD hosts.","title":"Horizon.Ops.BSD.Utils","ref":"Horizon.Ops.BSD.Utils.html"},{"type":"function","doc":"Assigns the application and options to a keyword list.","title":"Horizon.Ops.BSD.Utils.assigns/2","ref":"Horizon.Ops.BSD.Utils.html#assigns/2"},{"type":"function","doc":"Returns the options, with defaults inserted, for each release.\nIf no releases are defined in the mix.exs file, a default release returned.\n\nThis is the configured options for releases that you set for\neach release in `mix.exs` and can be inspected with `Mix.Project.config()[:releases]`.\nNot to be confused with the `%Mix.Release{}` struct that is passed to each `Step`.\nThe options here are stored in the `Mix.Release.options` field.","title":"Horizon.Ops.BSD.Utils.get_config_releases/0","ref":"Horizon.Ops.BSD.Utils.html#get_config_releases/0"},{"type":"function","doc":"","title":"Horizon.Ops.BSD.Utils.get_src_path/2","ref":"Horizon.Ops.BSD.Utils.html#get_src_path/2"},{"type":"function","doc":"Returns a tuple with the full source path and the target file name.\nFiles are referenced with an atom.\n\nNote that files in the `scripts` folder are expected to keep the same\nname from source to target. These static scripts will only be copied\nto the a `bin/` folder one time.","title":"Horizon.Ops.BSD.Utils.get_src_tgt/2","ref":"Horizon.Ops.BSD.Utils.html#get_src_tgt/2"},{"type":"function","doc":"iex> get_src_tgt(:stage, \"foo\")\n      {\"/path_to_horizon/priv/templates/bin/stage.sh.eex\", \"stage-foo.sh\"}","title":"Examples - Horizon.Ops.BSD.Utils.get_src_tgt/2","ref":"Horizon.Ops.BSD.Utils.html#get_src_tgt/2-examples"},{"type":"function","doc":"Validate that the `assets.setup.freebsd` alias exists in the mix.exs file.","title":"Horizon.Ops.BSD.Utils.warn_is_missing_freebsd_alias/1","ref":"Horizon.Ops.BSD.Utils.html#warn_is_missing_freebsd_alias/1"},{"type":"module","doc":"ADT for Horizon.Ops deployment targets","title":"Horizon.Ops.Target","ref":"Horizon.Ops.Target.html"},{"type":"function","doc":"","title":"Horizon.Ops.Target.is_static?/1","ref":"Horizon.Ops.Target.html#is_static?/1"},{"type":"function","doc":"","title":"Horizon.Ops.Target.is_template?/1","ref":"Horizon.Ops.Target.html#is_template?/1"},{"type":"type","doc":"","title":"Horizon.Ops.Target.content_type/0","ref":"Horizon.Ops.Target.html#t:content_type/0"},{"type":"type","doc":"","title":"Horizon.Ops.Target.key/0","ref":"Horizon.Ops.Target.html#t:key/0"},{"type":"type","doc":"","title":"Horizon.Ops.Target.t/0","ref":"Horizon.Ops.Target.html#t:t/0"},{"type":"module","doc":"Horizon.Ops is a tool for building and deploying\nElixir applications to FreeBSD hosts.","title":"Horizon.Ops.Utils","ref":"Horizon.Ops.Utils.html"},{"type":"function","doc":"Copy a file from source to target, overwriting if necessary.","title":"Horizon.Ops.Utils.copy_static_file/5","ref":"Horizon.Ops.Utils.html#copy_static_file/5"},{"type":"function","doc":"iex> safe_copy_file(\n      ...>   :horizon_helpers,\n      ...>   app,\n      ...>   overwrite,\n      ...>   false,\n      ...>   opts,\n      ...>   &Path.join(&2[:bin_path], &1)\n      ...> )","title":"Example - Horizon.Ops.Utils.copy_static_file/5","ref":"Horizon.Ops.Utils.html#copy_static_file/5-example"},{"type":"function","doc":"Create a file from a template.","title":"Horizon.Ops.Utils.create_file_from_template/7","ref":"Horizon.Ops.Utils.html#create_file_from_template/7"},{"type":"function","doc":"iex> create_file_from_template(\"source\", \"target\", true, false, %{}, &assigns/2, fn target,\n      ...>                                                                                opts ->\n      ...>   target\n      ...> end)","title":"Example - Horizon.Ops.Utils.create_file_from_template/7","ref":"Horizon.Ops.Utils.html#create_file_from_template/7-example"},{"type":"function","doc":"Safely copy a file from source to target.","title":"Horizon.Ops.Utils.safe_copy_file/4","ref":"Horizon.Ops.Utils.html#safe_copy_file/4"},{"type":"function","doc":"iex> safe_copy_file(\"source\", \"target\", true)\n        Created target","title":"Example - Horizon.Ops.Utils.safe_copy_file/4","ref":"Horizon.Ops.Utils.html#safe_copy_file/4-example"},{"type":"function","doc":"","title":"Horizon.Ops.Utils.safe_write/4","ref":"Horizon.Ops.Utils.html#safe_write/4"},{"type":"function","doc":"Validate the releases configuration for nil values.","title":"Horizon.Ops.Utils.validate_releases/1","ref":"Horizon.Ops.Utils.html#validate_releases/1"},{"type":"function","doc":"iex> validate_releases([..., build_user: nil, ..])\n  # => [\"phx_only -> build_user\", \"phx_only -> build_host\"]\n  releases","title":"Examples - Horizon.Ops.Utils.validate_releases/1","ref":"Horizon.Ops.Utils.html#validate_releases/1-examples"},{"type":"module","doc":"Project configuration for web based projects.\n\n  - `server_names` - List of recognized host names that can be received by nginx.\n  - `certificate` - The certificate to use for the project. Can be `nil`, `:self`, or `:letsencrypt`.\n  - `authenticator` - The authenticator to use for the project. Can be `nil` or a string, e.g. \"webroot\".\n  - `cert_path` - Overrides the path to the certificate file.\n  - `cert_key_path` - Overrides the path to the certificate key file.\n  - `letsencrypt_domain` - The domain to use for the letsencrypt certificate.\n  - `acme_challenge_path` - The root path to the acme challenge directory.\n  - `http_only` - If true, only the http clause will be defined.\n  - `servers` - List of servers to proxy to.","title":"Horizon.Project","ref":"Horizon.Project.html"},{"type":"module","doc":"#iex> projects = [\n        %Horizon.Project{\n          name: \"my_app1\",\n          server_names: [\"http-demo\"],\n          http_only: true,\n          # certificate: :letsencrypt,\n          # letsencrypt_domain: \"my_app\",\n          servers: [\n            # Verify PORT is same as in runtime.exs or env.sh.eex\n            %Horizon.Server{internal_ip: \"127.0.0.1\", port: 4000},\n            %Horizon.Server{internal_ip: \"10.0.0.5\", port: 4000}\n          ]\n        },\n        %Horizon.Project{\n          name: \"my_app2\",\n          server_names: [\"https-demo\"],\n          certificate: :self,\n          servers: [\n            # Verify PORT is same as in runtime.exs or env.sh.eex\n            %Horizon.Server{internal_ip: \"127.0.0.1\", port: 5000},\n            %Horizon.Server{internal_ip: \"10.0.0.5\", port: 5000}\n          ]\n        }\n      ]","title":"Examples - Horizon.Project","ref":"Horizon.Project.html#module-examples"},{"type":"function","doc":"Create a new project configuration.","title":"Horizon.Project.new/1","ref":"Horizon.Project.html#new/1"},{"type":"function","doc":"iex> Horizon.Project.new(name: \"my_project\", server_names: [\"my-domain.com\", \"also-mine.io\"])\n    %Horizon.Project{\n      name: \"my_project\",\n      server_names: [\"my-domain.com\", \"also-mine.io\"],\n      certificate: nil,\n      acme_challenge_path: nil,\n      http_only: false,\n      servers: []\n    }","title":"Examples - Horizon.Project.new/1","ref":"Horizon.Project.html#new/1-examples"},{"type":"type","doc":"","title":"Horizon.Project.cert/0","ref":"Horizon.Project.html#t:cert/0"},{"type":"type","doc":"","title":"Horizon.Project.t/0","ref":"Horizon.Project.html#t:t/0"},{"type":"module","doc":"Server configuration to define upstream servers for nginx.conf.","title":"Horizon.Server","ref":"Horizon.Server.html"},{"type":"function","doc":"Create a new server configuration.","title":"Horizon.Server.new/1","ref":"Horizon.Server.html#new/1"},{"type":"function","doc":"iex> Horizon.Server.new()\n    %Horizon.Server{internal_ip: \"127.0.0.1\", port: 4000}\n    iex> Horizon.Server.new(internal_ip: \"10.0.0.2\", port: 4001)\n    %Horizon.Server{internal_ip: \"10.0.0.2\", port: 4001}","title":"Examples - Horizon.Server.new/1","ref":"Horizon.Server.html#new/1-examples"},{"type":"type","doc":"","title":"Horizon.Server.t/0","ref":"Horizon.Server.html#t:t/0"},{"type":"task","doc":"This task checks for existing self-signed certificates created by `mix phx.gen.cert`.\nIf they don't exist it creates them.\n\nIt then copies the certificates to the `rel/overlays/cert` directory, and sets the file permissions to 600.\n\nThis allows you to access the deployed project webpage at its IP address over HTTPS without a domain name and letsencrypt certificate.","title":"mix horizon.gen.cert","ref":"Mix.Tasks.Horizon.Gen.Cert.html"},{"type":"task","doc":"mix horizon.gen.cert","title":"Examples - mix horizon.gen.cert","ref":"Mix.Tasks.Horizon.Gen.Cert.html#module-examples"},{"type":"task","doc":"Creates Horizon deploy scripts in `bin/` and `rel/` directories.","title":"mix horizon.init","ref":"Mix.Tasks.Horizon.Init.html"},{"type":"task","doc":"mix horizon.init [-y]","title":"Usage - mix horizon.init","ref":"Mix.Tasks.Horizon.Init.html#module-usage"},{"type":"task","doc":"* `-y` - Overwrite files without asking for confirmation.","title":"Options - mix horizon.init","ref":"Mix.Tasks.Horizon.Init.html#module-options"},{"type":"task","doc":"Horizon.Init creates scripts for deploying an Elixir application to a host.\nSee [`Horizon.Ops.Init`](Mix.Tasks.Horizon.Ops.Init.html) for the creation for additional helper applications.\n\nScripts are created in the `bin/` directory of the project by default,\nbut each release defined in `mix.exs` may override the default path\nby setting the `bin_path` option.\n\nThe run control script for each release is created in `rel/overlays/rc_d/` when `mix release` is run.","title":"Description - mix horizon.init","ref":"Mix.Tasks.Horizon.Init.html#module-description"},{"type":"task","doc":"Horizon.Init uses the `releases` configuration in `mix.exs` to customize the deployment scripts.\nThe available options are:\n\n- `bin_path`: `[bin/]`\n  - This is the directory where the scripts are created. If there is a `bin_path` for each release, scripts\n    associated with each release are copied to their respective bin directory.\n- `path`: `[/usr/local/ ]`\n  - This is the final destination of the release on the deploy host. This will be the same on deploy only hosts and the build host.\n- `build_host_ssh`: `[nil]`\n  - If not provided, `stage` script will use env var `BUILD_HOST_SSH`. Required for the `stage` script to copy the release to the build machine and for the `build` script.\n  - Example: `[user@]host`\n- `deploy_hosts_ssh`: `[[]]`\n  - If not provided, ssh hosts may be passed to the `deploy` script with the `-h` option.\n  - Example: `user@host1,user@host2`\n- `release_commands`: `[[]]`\n  - A list of commands to run after the release is copied to the deploy machine.\n  - These are typically zero arity commands defined in `release.ex`. For example, `[\"migrate\"]`.\n- `releases_path`: `[.releases]`\n  - The directory where releases are stored on the local host. The build script places the release tarball in this directory and the deploy\nscript copies the release from this directory.","title":"Customization - mix horizon.init","ref":"Mix.Tasks.Horizon.Init.html#module-customization"},{"type":"task","doc":"Running `mix horizon.init` creates several files in the `bin_path` directory.\nFor the project `my_app`, these files include:\n\n- `bin/horizon_helpers.sh`\n- `bin/stage-my_app.sh`\n- `bin/build-my_app.sh`\n- `bin/build_script-my_app.sh`\n- `bin/deploy-my_app.sh`\n- `bin/deploy_script-my_app.sh`\n\nIf you have multiple releases, a `stage`, `build` and `deploy` script\nis created for each release.\nFor example, imagine you have releases `app_web` and `app_worker`.\nHorizon.Ops.Init will create\n\n- `bin/horizon_helpers.sh`\n- `bin/stage-app_web.sh`\n- `bin/build-app_web.sh`\n- `bin/build_script-app_web.sh`\n- `bin/deploy-app_web.sh`\n- `bin/deploy_script-app_web.sh`\n- `bin/stage-app_worker.sh`\n- `bin/build-app_worker.sh`\n- `bin/build_script-app_worker.sh`\n- `bin/deploy-app_worker.sh`\n- `bin/deploy_script-app_worker.sh`","title":"Files Created - mix horizon.init","ref":"Mix.Tasks.Horizon.Init.html#module-files-created"},{"type":"task","doc":"Creates Horizon.Ops helper scripts in `ops/bin/` directory.","title":"mix horizon.ops.init","ref":"Mix.Tasks.Horizon.Ops.Init.html"},{"type":"task","doc":"mix horizon.ops.init [-y] [path]","title":"Usage - mix horizon.ops.init","ref":"Mix.Tasks.Horizon.Ops.Init.html#module-usage"},{"type":"task","doc":"* `-y` - Overwrite files without asking for confirmation.\n  * `path` - The path to the directory where the scripts are created. Default is `ops/bin`.","title":"Options - mix horizon.ops.init","ref":"Mix.Tasks.Horizon.Ops.Init.html#module-options"},{"type":"task","doc":"Horizon.Ops.Init creates several scripts for managing deployments of hosts and data.\n\nRunning `mix horizon.ops.init` creates several files in the `ops/bin/` directory.\nThese files include:\n\n- `bsd_install.sh`\n- `bsd_install_args.sh`\n- `bsd_install_script.sh`\n- `horizon_helpers.sh`\n- `add_certbot_crontab.sh`\n- `backup_databases.sh`\n- `backup_databases_over_ssh.sh`\n- `restore_database.sh`\n- `update_database_owner.sh`\n- `turn_on_postgres_access.sh`\n- `turn_off_postgres_access.sh`\n- `turn_on_user_access.sh`\n- `turn_off_user_access.sh`\n- `freebsd_setup.sh`\n- `zfs_snapshot.sh`","title":"Description - mix horizon.ops.init","ref":"Mix.Tasks.Horizon.Ops.Init.html#module-description"},{"type":"extras","doc":"# Horizon\n\nWelcome to **Horizon**, an ops library for **deploying Elixir/Phoenix\nprojects** to FreeBSD hosts.","title":"Horizon","ref":"readme.html"},{"type":"extras","doc":"- Only SSH access is required for administration\n- Uses install files so you can version manage host configuration\n- Installs Elixir, Erlang, Postgres and Postgres databases\n- Full Postgres install with a single command\n- Tools to setup Postgres backup server\n- Simplifies FreeBSD management on ZFS file systems\n- Can deploy to bare-metal or VM hosts","title":"Features - Horizon","ref":"readme.html#features"},{"type":"extras","doc":"This guide is intended to help you get started with Horizon for deploying your Elixir/Phoenix application to FreeBSD hosts.\nFollow the installation instructions on this page to configure your existing project to use Horizon for deployment.\n\nThen follow the guides below to setup your host servers and deploy your Elixir/Phoenix application with Horizon.\n\n- [Deploying with Horizon](deploying-with-horizon.html)\n- [Horizon Ops Scripts](horizon-helper-scripts.html)\n- [Sample Host Configurations](sample-host-configurations.html)\n- [Proxy Configuration](proxy-conf.html)","title":"Getting Started - Horizon","ref":"readme.html#getting-started"},{"type":"extras","doc":"- [FreeBSD Template Setup](freebsd-template-setup.html)\n- [Hetzner Cloud Setup Guide](hetzner-cloud.html)\n- [Hetzner Cloud Host Instantiation](hetzner-cloud-host-instantiation.html)\n- [FreeBSD Installation](freebsd-install.html)\n- [Creating a FreeBSD VM on Proxmox](proxmox.html)","title":"Additional Guides and Resources - Horizon","ref":"readme.html#additional-guides-and-resources"},{"type":"extras","doc":"Add `horizon` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n [\n    {:horizon, \"~> 0.2\", runtime: false}\n ]\nend\n```\n\nAfter adding `horizon` to your list of dependencies, run:\n\n```shell\nmix deps.get\n```\n\nUse the following instructions to configure your application so Horizon can \n`stage`, `build` and `deploy` your applications.\n\n- [Define a Release in `mix.exs`](#configuring-your-project-for-horizon)\n- [Add Tailwind to your `assets.setup`](#configuring-tailwind-for-freebsd)\n- [Run `mix horizon.init`](#running-mix-horizon-init)","title":"Installation - Horizon","ref":"readme.html#installation"},{"type":"extras","doc":"Horizon builds scripts for each release in your project to `stage`, `build`, and `deploy` your Elixir/Phoenix app.","title":"Configuring your project for Horizon - Horizon","ref":"readme.html#configuring-your-project-for-horizon"},{"type":"extras","doc":"If you haven't defined a release, add one to your `mix.exs` file. \n\nThe simplest release configuration looks like:\n```elixir\n  def project do\n    [\n      app: :my_app,\n      ...,\n      releases: [my_app: []]\n      ...\n    ]\n  end\n```\n\nIn general however, you will want configure access information for your build and deploy hosts.\nThe example below also configures [steps](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-steps) \nto set the [default release values for a FreeBSD system\nand to create the run control script](Horizon.Ops.BSD.Step.html) in the `rel/overlays/rc_d/` folder. \n\n\n```elixir\n  def project do\n    [\n      app: :my_app,\n      ...,\n      releases: [\n        my_app: [\n          applications: [runtime_tools: :permanent],\n          include_executables_for: [:unix],\n          build_host_ssh: System.get_env(\"BUILD_HOST_SSH\"),\n          deploy_hosts_ssh: System.get_env(\"DEPLOY_HOSTS_SSH\"),\n          # app_path: \"/usr/local/my_app\",\n          # bin_path: \"bin\",\n          # build_path: \"/usr/local/opt/my_app/build\",\n          # release_commands: [],\n          # releases_path: \".releases\",\n          steps: [\n            &Horizon.Ops.BSD.Step.setup/1,\n            :assemble,\n            :tar\n          ],\n        ]\n      ]\n    end\n```\n> `assemble` is required to create your release target. `tar` is needed to build a tarball for deployment.","title":"Add a Release to your `mix.exs` file - Horizon","ref":"readme.html#add-a-release-to-your-mix-exs-file"},{"type":"extras","doc":"If you are using a default `config/runtime.exs` file, it ships with a default IPv6 listen address.\nFor the examples in this guide you will need to configure it for IPv4.\n\n```elixir\n  config :my_app1, MyApp1Web.Endpoint,\n    url: [host: host, port: 443, scheme: \"https\"],\n    http: [\n      # Enable IPv6 and bind on all interfaces.\n      # Set it to  {0, 0, 0, 0, 0, 0, 0, 1} for local network only access.\n      # See the documentation on https://hexdocs.pm/bandit/Bandit.html#t:options/0\n      # for details about using IPv6 vs IPv4 and loopback vs public addresses.\n      ip: {0, 0, 0, 0},\n      port: port\n    ],\n```","title":"Configure Runtime.exs for IPv4 - Horizon","ref":"readme.html#configure-runtime-exs-for-ipv4"},{"type":"extras","doc":"You may also need to configure the host and port in your `config/runtime.exs` file.\nThe host `demo-web` in this example is the name used in your browser to access your website.\nThis can be an actual domain name,  an IP address or an alias in your `/etc/hosts` file.\n\nThe `port` is the port number your Phoenix application will listen on. \nIf you are running multiple applications in your Horizon setup, each application will need a unique port number.\n\n```elixir\n  host = System.get_env(\"PHX_HOST\") || \"demo-web1\"\n  port = String.to_integer(System.get_env(\"PORT\") || \"4000\")\n```","title":"Configure Host and Port in Runtime.exs - Horizon","ref":"readme.html#configure-host-and-port-in-runtime-exs"},{"type":"extras","doc":"The default mix alias `assets.setup` is:\n\n```\n\"assets.setup\": [\n   \"tailwind.install --if-missing\",\n   \"esbuild.install --if-missing\"\n],\n```\n\nHowever, a tailwind download for FreeBSD is not currently provided by the Tailwind project.\nThis is resolved by passing a URL to `tailwind.install` from which to download the tailwind executable.\n\n**Add the `\"assets.setup.freebsd\"` mix alias to your `mix.exs` file.**\n\n```\n@tailwindcss_freebsd_x64 \"https://people.freebsd.org/~dch/pub/tailwind/v$version/tailwindcss-$target\"\n\n...\ndefp aliases do\n  [\n    ...\n    \"assets.setup\": [\"tailwind.install --if-missing\", \"esbuild.install --if-missing\"],\n    \"assets.setup.freebsd\": [\n      \"tailwind.install #{@tailwindcss_freebsd_x64}\",\n      \"esbuild.install --if-missing\"\n    ],\n    ...\n  ]\nend\n```","title":"Configuring Tailwind for FreeBSD - Horizon","ref":"readme.html#configuring-tailwind-for-freebsd"},{"type":"extras","doc":"Running `mix horizon.init` creates scripts for each of the`releases` defined in your mix project. FreeBSD customary defaults are used for installation paths and are added when you include `&Horizon.Ops.BSD.Step.setup/1` as the first `step` of each release.\n\nRunning this script will provide instructions for creating a mix alias `\"assets.setup.freebsd\"` (if not already configured) that will install `esbuild` and `tailwind` on your build host.\n\n```shell\n❯ mix horizon.init\nCreated   bin/horizon_helpers.sh\nCreated   bin/stage-my_app1.sh\nCreated   bin/build-my_app1.sh\nCreated   bin/build_script-my_app1.sh\nCreated   bin/deploy-my_app1.sh\nCreated   bin/deploy_script-my_app1.sh\n```\n\n> **You must run `mix horizon.init` to update your scripts every time you change a `release` in `mix.exs`.**","title":"Horizon Script Generation - Horizon","ref":"readme.html#horizon-script-generation"},{"type":"extras","doc":"Running `mix release.init` creates a `rel/` directory that contains the file `env.sh.eex`. \n**The simplest way to manage env vars is to add them directly to `rel/env.sh.eex`:**\n\n```\n...\nexport PHX_SERVER=true\nexport SECRET_KEY_BASE=O7Delm59ZMLKlSh80ZnrBIhZmf6sz1NVMhbAofIxYNNZUqnkaa9SnizAA1jpxDl6\nexport PGHOSTADDR=10.0.0.3\nexport PHX_HOST=demo-web1\n\nexport DATABASE_URL=ecto://451f8d75-a808-11ef-8e9c-e1aff46a3315:0462ae2ea0e180b4beb0558bf5baec29e87c1ec9bd4f5c6c@$PGHOSTADDR/my_app1_prod\n...\n```\n\nUpdate the `env.sh.eex` file with the environment variables required for your application.\n\n> You can generate a secret key with:\n> ```\n> mix phx.gen.secret\n> ```","title":"Configuring Env vars for releases - Horizon","ref":"readme.html#configuring-env-vars-for-releases"},{"type":"extras","doc":"If you are using a `.env` file for environment variables, one method is just to add it to your `env.sh.eex` file:\n\n```\ncat .env >> rel/env.sh.eex\n```\n\nOr, reference the `.env` file from within `env.sh.eex`.\n\n```shell\nenv_file=/usr/local/my_app/.env\nchmod 600 $env_file\n. $env_file\n```\n\nIf using this method, you will need to add `.env` to your deploy artifacts. \nOne solution is to copy the .env file to your `overlays` folder in `rel/overlays/.env`.\n\n> This method may be insufficient since the same `.env` file is shared for all deploy versions. \nWriting a [`step`](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-steps) or a more sophisticated overlay that is version dependent may be required if not using `env.sh.eex`.\n\n\nSource code is licensed under the [BSD 3-Clause License](LICENSE.md).\n\nSee the [Getting Started](#getting-started) section for more information on configuring your project for Horizon.\n\n\n---","title":"Alternative: Importing existing `.env` file - Horizon","ref":"readme.html#alternative-importing-existing-env-file"},{"type":"extras","doc":"# Deploying with Horizon\n\nThis guide walks you through using Horizon to deploy your ElixirPhoenix application to production hosts.\n\nWe will cover host configuration and the deployment process to release your web application to production.\n\n> This guide assumes you have hosts with FreeBSD installed, names and/or addresses for those hosts, and passwordless access to the hosts. See the installation resources for more information on setting up your hosts.","title":"Deploying with Horizon","ref":"deploying-with-horizon.html"},{"type":"extras","doc":"This guide will walk you through configuring a four server topology.\nWe will use the names `demo-web1`, `demo-web2`, `demo-build`, and `demo-pg1`, and `demo-pg2` for the web, build, and database hosts respectively.\n\nThe application cluster uses `demo-web1` as a web host **and** the `nginx` reverse-proxy.\nThe `web` hosts `demo-web1` and `demo-web2` are connected to a PostgreSQL database server, `demo-pg1`. \nThe `demo-pg1` server is backed up by `demo-pg2` using a rolling snapshot backup strategy.\n\nFinally, the fifth server is `build` host `demo-build` that is used to build the release tarball\nthat is deployed to the `demo-web1` and `demo-web2` hosts.\n\nThe topology is illustrated below.\n\n```mermaid\ngraph BT\n    linkStyle default stroke-width:3px\n    user1[Web User 1] -->|Browser Connection| nginx[Nginx on web1]\n    user2[Web User 2] -->|Browser Connection| nginx\n    nginx -.->|Upstream Connection| web1[Web Server web1]\n    nginx -.->|Upstream Connection| web2[Web Server web2]\n    web1 -->|Database Access| pg1[PostgreSQL pg1]\n    web2 -->|Database Access| pg1\n    pg2[PostgreSQL pg2] -->|Rolling Snapshot Backup| pg1\n\n    subgraph web1_components [Web1 Host]\n        direction TB\n        style web1_components stroke-dasharray: 5 5\n        nginx\n        web1\n    end\n```","title":"Web Cluster Topology - Deploying with Horizon","ref":"deploying-with-horizon.html#web-cluster-topology"},{"type":"extras","doc":"Horizon has a suite of tools for deploying your Elixir/Phoenix application to a FreeBSD host. These tools are installed with:\n\n```\nmix horizon.ops.init\n```\nThe Horizon ops scripts are not specific to your application; therefore, you can install them anywhere on your system.\n\nThe default install location is `ops/bin` inside your project, but you may want to install them in `$HOME/bin` so they are more generally available.\n\n```shell\nmix horizon.ops.init ~/bin\n```\n\nAnd update your path if needed:\n```shell\nexport PATH=$PATH:~/bin\n```","title":"Installing Horizon Ops Tools - Deploying with Horizon","ref":"deploying-with-horizon.html#installing-horizon-ops-tools"},{"type":"extras","doc":"If you have just followed the [host instantiation instructions](hetzner-cloud-host-instantiation.html) and created five VMs, you will need to assign host names to each VM and, for convenience, add them to your `/etc/hosts` file.\n\nDefine the host names in your `/etc/hosts` file with the public IP addresses of each host.\n\nThe IP addresses for our demo project were:\n\n```shell\ncat /etc/hosts\n...","title":"Naming Hosts and Configuring LAN - Deploying with Horizon","ref":"deploying-with-horizon.html#naming-hosts-and-configuring-lan"},{"type":"extras","doc":"178.156.153.24\tdemo-build\n5.161.249.144\tdemo-web1\n178.156.153.23\tdemo-web2\n178.156.153.21\tdemo-pg1\n178.156.153.22\tdemo-pg2\n```","title":"Demo - Deploying with Horizon","ref":"deploying-with-horizon.html#demo"},{"type":"extras","doc":"With host name aliases set, you can set the hostname of each host.\n\n```shell\nssh admin@demo-web1 \"doas hostname demo-web1; doas sysrc hostname=demo-web1\"\nssh admin@demo-web2 \"doas hostname demo-web2; doas sysrc hostname=demo-web2\"\nssh admin@demo-pg1 \"doas hostname demo-pg1; doas sysrc hostname=demo-pg1\"\nssh admin@demo-pg2 \"doas hostname demo-pg2; doas sysrc hostname=demo-pg2\"\nssh admin@demo-build \"doas hostname demo-build; doas sysrc hostname=demo-build\"\n```","title":"Set Hostnames - Deploying with Horizon","ref":"deploying-with-horizon.html#set-hostnames"},{"type":"extras","doc":"Hetzner Cloud VMs are configured with a private network interface on `vtnet1`.\nWe will configure the `vtnet1` interface on each host to use DHCP.\n\n```shell\nssh admin@demo-web1 \"doas sysrc ifconfig_vtnet1=DHCP\"\nssh admin@demo-web2 \"doas sysrc ifconfig_vtnet1=DHCP\"\nssh admin@demo-pg1 \"doas sysrc ifconfig_vtnet1=DHCP\"\nssh admin@demo-pg2 \"doas sysrc ifconfig_vtnet1=DHCP\"\nssh admin@demo-build \"doas sysrc ifconfig_vtnet1=DHCP\"\n```\n\nNote that on Hetzner Cloud, the DHCP server assigns a netmask of `0xFFFF`.\nThis means that there is no LAN in the traditional sense and no `arp` requests.\nInstead, traffic sent to a another host on the \"LAN\" is handled by the gateway router.\n\nThis will come into play when configuring the host-based authentication for the database.\n\n```shell\n$ netstat -rn\nRouting tables\n\nInternet:\nDestination        Gateway            Flags     Netif Expire\ndefault            172.31.1.1         UGS      vtnet0\n5.161.249.144      link#3             UH          lo0\n10.0.0.0/16        10.0.0.1           UGS      vtnet1\n10.0.0.1           link#2             UHS      vtnet1\n10.0.0.2           link#3             UH          lo0\n127.0.0.1          link#3             UH          lo0\n```","title":"Configure LAN - Deploying with Horizon","ref":"deploying-with-horizon.html#configure-lan"},{"type":"extras","doc":"Each type of host has different requirements. \nYou can use Horizon Ops `bsd_install.sh` and a config file to set up each host according its needs. \nWe'll look at each type of host and the configuration needed -- `web`, `postgres`, `postgres-backup`, and `build`.\n\n[Sample configuration files are provided for each host type](sample-host-configurations.html).","title":"Configuring Hosts - Deploying with Horizon","ref":"deploying-with-horizon.html#configuring-hosts"},{"type":"extras","doc":"A web host has minimal configuration because we ship the Erlang runtime in the deployments. This allows you to update the Elixir version on deployments.\n\nFor this example, the web host is also running the reverse proxy, we we'll install the `nginx` application, start the `nginx` service and install `certbot` for future certification configuration.\n\nCreate a file named `web+proxy.conf` with the following content:\n\n```\npkg:nginx\nservice:nginx\npkg:py311-certbot\n```\n\nand configure your web host with:\n\n```\nbsd_install.sh admin@demo-web1 web+proxy.conf\n```\n\nIf your are horizontally scaling your web app topology with additional web hosts, you only need to install specific applications that they may need. Typically, other web hosts will not need proxies, certificates, or other services.","title":"Configure Web Hosts - Deploying with Horizon","ref":"deploying-with-horizon.html#configure-web-hosts"},{"type":"extras","doc":"Your postgres host serves as a database for all of the web hosts. \nFor performance reasons, you may use a larger cloud server or a dedicated server for your database.\n\nThis example installs Postgres on `demo-pg1`.\n\nThere are four installation steps we will use to stand up a Postgres database:\n\n1. Install the version of postgres server and contrib library of your choice\n2. Configure zfs for database snapshots.\n3. Initialize postgres. This configures `postgresql.conf`, `pg_hba.conf` and logging. Logging is at `/var/log/postgresql.log`\n4. Create a database. Creating a database creates a user/password, and updates `pg_hba.conf`. You can also choose the locale and `ctype` of the database.\n\nThree postgres encoding options are available:\n1. postgres-db-c_mixed_utf8\n2. postgres-db-c_utf8_full\n3. postgres-db-us_utf8_full\n\nThe most common option is `c_mixed_utf8` which sorts with byte order for speed, but encodes with `UTF-8`.\nThis format, while common, will not sort `UTF-8` characters as desired.\n\nIn this example, we install `postgresql17-server` and `postgresql17-contrib` and initialize the database with the `c_mixed_utf8` encoding.\nNote that we also initialize `zfs` before initializing the database. \nThe `zfs` initialization is required for the backup host to take snapshots of the database.\n\nCreate the file `postgres.conf` with:\n```\npkg:postgresql17-server\npkg:postgresql17-contrib\npostgres.zfs-init\npostgres.init\n\n# Encode with UTF8 and sort with byte order\npostgres.db:c_mixed_utf8:my_app1_prod\n```\n\nand install with:\n\n```\nbsd_install.sh admin@demo-pg1 postgres.conf\n```\n\nRunning this command generated the following output:\n\n```shell\n...\nCREATE ROLE\n[SUCCESS] User 451f8d75-a808-11ef-8e9c-e1aff46a3315 created successfully.\nCREATE DATABASE\n[SUCCESS] Database my_app1_prod created successfully.\n[SUCCESS] Added user 451f8d75-a808-11ef-8e9c-e1aff46a3315@my_app1_prod to pg_hba.conf.\n[SUCCESS] Postgres reloaded successfully.\n[SUCCESS] Username: 451f8d75-a808-11ef-8e9c-e1aff46a3315\n[SUCCESS] Password: 0462ae2ea0e180b4beb0558bf5baec29e87c1ec9bd4f5c6c\n[SUCCESS] Database: my_app1_prod\n[WARN] Record these credentials in a secure location.\n[WARN] You will not be able to retrieve the password later.\n```\n\nSave the `username` and `password` in a secure location. You will need them to configure your Elixir/Phoenix application.","title":"Configure Postgres Host - Deploying with Horizon","ref":"deploying-with-horizon.html#configure-postgres-host"},{"type":"extras","doc":"To configure a backup host, you can use a similar configuration as the Postgres host. The backup host will have the same version of Postgres, but we will not initialize postgres with `postgres.init` nor create a database.\n\nCreate the file `#postgres-backup.conf` with:\n```\npkg:postgresql17-server\npkg:postgresql17-contrib\npostgres.zfs-init\n```\n\nand install with:\n\n```\nbsd_install.sh admin@demo-pg2 postgres-backup.conf\n```\n\n#### Install Backup Scripts and Cron Jobs\n\nTo finalize the backup configuration you will need to:\n- Copy the `zfs_backup.sh` script to the backup host. This script is used to backup the Postgres database to a ZFS volume.\n- Ensure `demo-pg2` had passwordless access to `demo-pg1`.\n- Add a crontab to run `zfs_snapshot.sh`\n\nFirst, copy the `zfs_snapshot.sh` script to the backup host. Remember to copy `zfs_snapshot.sh` from the location where you installed the Horizon ops scripts.\n\n```shell\nssh admin@demo-pg2 \"mkdir -p bin\"\nscp ~/bin/zfs_snapshot.sh admin@demo-pg2:bin\n```\n\nYou will likely need to add an ssh key to `demo-pg2` and copy its public key to `demo-pg1` and verify passwordless access to `demo-pg1` from `demo-pg2`.\n\n```shell\nssh admin@demo-pg2 \"ssh-keygen\"\n```\n\n```shell\nscp admin@demo-pg2:.ssh/id_rsa.pub /tmp\nscp /tmp/id_rsa.pub admin@demo-pg1:/tmp\nssh admin@demo-pg1 \"cat /tmp/id_rsa.pub >> .ssh/authorized_keys\"\n```\n\nBefore adding the cronjob, log into `demo-pg2` and verify that you can ssh to `demo-pg1` without a password. For this example we added a host to `/etc/hosts` to make it easier to reference the `demo-pg1`.\n\n```terminal\n$ tail -1 /etc/hosts\n10.0.0.3\tpg1\n```\n\nThis command below assumes `pg1` is in `/etc/hosts` and you can ssh to `admin@pg1` from `demo-pg2` without a password.\n\n```shell\nssh admin@demo-pg2 \"CRON_JOB='*/30 * * * * /home/admin/bin/zfs_snapshot.sh admin@pg1 >> /var/log/zfs_snapshot.log 2>&1'; (crontab -l 2>/dev/null | grep -Fq \\\"\\$CRON_JOB\\\") || (crontab -l 2>/dev/null; echo \\\"\\$CRON_JOB\\\") | crontab -\"\n```\n\n#### Verify the Backup\n\nYou can verify the backup by connecting to `demo-pg2` and running the backup command:\n\nRunning the backup on our demo produced the following output:\n\n```shell\nssh admin@demo-pg2 \nbin/zfs_snapshot.sh admin@pg1\nzfs list -t snapshot\n```\n\n```shell\n$ ./bin/zfs_snapshot.sh pg1\nRunning PostgreSQL backup and creating ZFS snapshot...\n pg_backup_start\n-----------------\n 0/2000028\n(1 row)\n\nERROR:  backup is not in progress\nHINT:  Did you call pg_backup_start()?\nPostgreSQL backup and ZFS snapshot completed.\nNo common snapshot found, or no previous snapshots. Sending full snapshot.\nSending full rolling snapshot zroot/var/db_postgres@rolling-20241121080548 to controlling host...\nfull send of zroot/var/db_postgres@rolling-20241121080548 estimated size is 13.8M\ntotal estimated size is 13.8M\nTIME        SENT   SNAPSHOT zroot/var/db_postgres@rolling-20241121080548\nCleaning up old rolling snapshots...\nNot enough snapshots for cleanup. Current count:        1, required: 48\nSnapshot process completed.\nadmin@demo-pg2:~ $ zfs list -t snapshot\nNAME                                           USED  AVAIL  REFER  MOUNTPOINT\nzroot/ROOT/default@2024-11-19-11:35:22-0       394M      -  2.10G  -\nzroot/ROOT/default@initial-setup               500K      -  2.12G  -\nzroot/var/db_postgres@rolling-20241121080548    56K      -  14.3M  -\n```\n\n> The \"ERROR:  backup is not in progress\" message is expected since `pg_backup_start` is run at the start of the backup, but a backup was never requested from postgres. Instead, a physical backup via `zfs` is run and this error is generated when `pg_backup_stop` is called.\n\nIt's a good idea to also validate that the log file was created and contains the expected output.\n\n```shell\nbin/zfs_snapshot.sh admin@pg1 >> /var/log/zfs_snapshot.log\ntail /var/log/zfs_snapshot.log\nzfs list -t snapshot\n```","title":"Configure Postgres Backup Host - Deploying with Horizon","ref":"deploying-with-horizon.html#configure-postgres-backup-host"},{"type":"extras","doc":"To prepare a build host, you'll need to install the Erlang runtime and Elixir.\nFor simplicity, we maintain only one version of each on a `build` host.\n\nWe use the latest packaged (pkg) version of Erlang for convenience but build our own version of Elixir.\nThe build host is responsible for creating the release tarball.\n\nCreate a file named `build.conf` with the following content:\n\n```\npkg:ca_root_nss\npkg:gcc\npkg:rsync\npkg:gmake\npkg:git\npkg:erlang-runtime27\n\n# Set the path to erlang so we can install elixir\npath:/usr/local/lib/erlang27/bin\n\nelixir:1.17.3\n```\n\nTo install the necessary packages and configure the host, run:\n\n```\nbsd_install.sh admin@demo-build build.conf\n```","title":"Configure Build Host - Deploying with Horizon","ref":"deploying-with-horizon.html#configure-build-host"},{"type":"extras","doc":"To Configure `nginx` we use `Horizon.NginxConfig` module to send the configuration to the proxy host.\n\nIn the following example, we configure the `nginx` server to proxy requests to the `my_app1` application running on `demo-web1`. \n\n```elixir\nuser=\"admin\"\nhost=\"demo-web1\"\n\nprojects = [\n  %Horizon.Project{\n    name: \"my_app1\",\n    server_names: [\"demo-web1\"],\n    http_only: true,\n    # certificate: :letsencrypt,\n    # letsencrypt_domain: \"my_app.com\",\n    servers: [\n      # Verify PORT is same as in runtime.exs or env.sh.eex\n      %Horizon.Server{internal_ip: \"10.0.0.2\", port: 4000},\n      %Horizon.Server{internal_ip: \"10.0.0.5\", port: 4000}\n    ]\n  }\n]\nHorizon.NginxConfig.send(projects, user, host, action: :restart)\n```\n\nIf you want to add a second application `my_app2` to the same application cluster, you can add it to the `projects` list and call `Horizon.NginxConfig.send/4` again.\n\n> When running multiple applications on the same host, ensure that the ports are unique for each application.\n\nHere is an example of configuring two applications on the same host.\n`my_app1` is deployed with `http_only` and `my_app2` uses a self-signed cert.\n\n```shell\n# in my_app2\nmix horizon.gen.cert\n```\n\nIn a Livebook cell, you can run the following code to configure the `nginx` server to proxy requests to the `my_app1` and `my_app2` applications running on `demo-web1` and `demo-web2`.\n\n```elixir\nuser=\"admin\"\nhost=\"demo-web1\"\n\nprojects = [\n  %Horizon.Project{\n    name: \"my_app1\",\n    server_names: [\"my-app1\"],\n    http_only: true,\n    # certificate: :letsencrypt,\n    # letsencrypt_domain: \"my_app\",\n    servers: [\n      # Verify PORT is same as in runtime.exs or env.sh.eex\n      %Horizon.Server{internal_ip: \"127.0.0.1\", port: 4000},\n      %Horizon.Server{internal_ip: \"10.0.0.5\", port: 4000}\n    ]\n  },\n  %Horizon.Project{\n    name: \"my_app2\",\n    server_names: [\"my-app2\"],\n    certificate: :self,\n    servers: [\n      # Verify PORT is same as in runtime.exs or env.sh.eex\n      %Horizon.Server{internal_ip: \"10.0.0.2\", port: 5000},\n      %Horizon.Server{internal_ip: \"10.0.0.5\", port: 5000}\n    ]\n  }\n]\nHorizon.NginxConfig.send(projects, user, host, action: :restart)\n```\n\nIf you have open ports on your firewall, you will be able to access each application by the server name and port number when using the `http_only: true` option.\n\nCertificate configuration is covered in the next section.","title":"Configure Reverse Proxy - Deploying with Horizon","ref":"deploying-with-horizon.html#configure-reverse-proxy"},{"type":"extras","doc":"There are several options to consider when configuring certificates for your web host. \n- no certificates - use http only for testing\n- self-signed certificates - use for testing when you don't have a domain name\n- acme certificates - per domain certificates\n- wildcard certificates - for multiple subdomains. requires DNS configuration","title":"Configure Certificates - Deploying with Horizon","ref":"deploying-with-horizon.html#configure-certificates"},{"type":"extras","doc":"You can install self-signed certificates using Horizon's mix task\n```shell\nmix horizon.gen.cert\n```\n\nThis will run `mix phx.gen.cert` if no certificates are found in the `priv/cert` directory and will copy the certificates to the `rel/overlays/cert` directory. This will allow you to use the certificates in your release when you don't have a domain name.\n\n#### ACME Certificates\n\nYou can generate certificates using `certbot` with the following command on the `demo-web1` host:\n\n```shell\ndoas certbot certonly \\\n  --dry-run --webroot \\\n  --webroot-path /usr/local/my_app1 \\\n  --rsa-key-size 4096 \\\n  --email me@example.com \\\n  --agree-tos \\\n  --non-interactive \\\n   -d my_app1.com\n```\n\nRemove the `--dry-run` option to generate a real certificate.\n\n#### Wildcard Certificates\n\nIf you have multiple subdomains, you can use a wildcard certificate.\n`certbot` supports multiple DNS providers to automate the process.\n\nOn the `demo-web1` host, you can get a list of the supported providers:\n\n```shell\n $ pkg search certbot\npy311-certbot-2.11.0,1         Let's Encrypt client\npy311-certbot-apache-2.11.0    Apache plugin for Certbot\npy311-certbot-dns-cloudflare-2.11.0_1 Cloudflare DNS plugin for Certbot\npy311-certbot-dns-cpanel-0.4.0 CPanel DNS Authenticator plugin for Certbot\npy311-certbot-dns-digitalocean-2.11.0 DigitalOcean DNS Authenticator plugin for Certbot\npy311-certbot-dns-dnsimple-2.11.0 DNSimple DNS Authenticator plugin for Certbot\npy311-certbot-dns-dnsmadeeasy-2.11.0 DNS Made Easy DNS Authenticator plugin for Certbot\npy311-certbot-dns-gandi-1.5.0  Gandi LiveDNS plugin for Certbot\npy311-certbot-dns-gehirn-2.11.0 Gehirn Infrastructure Service DNS Authenticator plugin for Certbot\npy311-certbot-dns-google-2.11.0 Google Cloud DNS Authenticator plugin for Certbot\npy311-certbot-dns-linode-2.11.0 Linode DNS Authenticator plugin for Certbot\npy311-certbot-dns-luadns-2.11.0 LuaDNS Authenticator plugin for Certbot\npy311-certbot-dns-nsone-2.11.0 NS1 DNS Authenticator plugin for Certbot\npy311-certbot-dns-ovh-2.11.0   OVH DNS Authenticator plugin for Certbot\npy311-certbot-dns-powerdns-0.2.1_1 PowerDNS DNS Authenticator plugin for Certbot\npy311-certbot-dns-rfc2136-2.11.0 RFC 2136 DNS Authenticator plugin for Certbot\npy311-certbot-dns-route53-2.11.0 Route53 DNS Authenticator plugin for Certbot\npy311-certbot-dns-sakuracloud-2.11.0 Sakura Cloud DNS Authenticator plugin for Certbot\npy311-certbot-dns-standalone-1.1 Standalone DNS Authenticator plugin for Certbot\npy311-certbot-nginx-2.11.0     NGINX plugin for Certbot\n```\n\nUsing DNSimple as an example and following their instructions, I created a secrets folder and added an API token:\n\n```shell\nmkdir -p ~/.secrets/certbot\necho \"dns_dnsimple_token = YOUR_DNSIMPLE_API_TOKEN\" >> ~/.secrets/certbot/dnsimple.ini\nchmod 600 ~/.secrets/certbot/dnsimple.ini\n```\n\nThen I ran the following command to generate a wildcard certificate:\n\n```shell\n  doas certbot certonly \\\n  --dns-dnsimple \\\n  --dns-dnsimple-credentials ~/.secrets/certbot/dnsimple.ini \\\n  --dns-dnsimple-propagation-seconds 60 \\\n  --non-interactive \\\n  --agree-tos \\\n  --email me@example.com \\\n  -d \"example.com\" -d \"*.example.com\"\n```\n\n#### Updating Nginx Configuration with Certificates\n\nWhen you have your certificates, you can update the `nginx` configuration to use them.\n\nIf you are using standard Let's Encrypt certificates, you can simplify configuration by providing the domain name and the certificate type:\n```elixir\niex> projects =[%Horizon.Project{\n    name: \"my_app1\",\n    server_names: [\"demo-web1\"],\n    certificate: :letsencrypt,\n    letsencrypt_domain: \"example.com\",\n    #acme_challenge_path: \"custom_path\",\n    servers: [\n      # Verify PORT is same as in runtime.exs or env.sh.eex\n      %Horizon.Server{internal_ip: \"127.0.0.1\", port: 4000},\n      %Horizon.Server{internal_ip: \"10.0.0.5\", port: 4000}\n    ]\n  }]\n  Horizon.NginxConfig.send(projects, user, host, action: :restart)\n```\n\nYou can also specify the path to the acme challenge if you have a custom path by setting the `acme_challenge_path` explicitly.\n\n#### Renewing Certificates\n\nLetsencrypt certificates have a lifespan of 90 days and therefore need to be renewed on a periodic basis.\n\nCertificate renewal is done with `certbot renew`. You can use the Horizon script `add_certbot_crontab.sh` to schedule a cron job to check twice daily if certificates need to be renewed. If the certificates are due for renewal, the cron job will renew them and reload the `nginx` service.\n\nRun this script to add a cron job to `demo-web1`:\n\n```shell\n$ bin/add_certbot_crontab.sh admin@demo-web1\nCron job added successfully.\n```\n\nThis script will add a cron job that looks like this:\n\n```shell\n0 0,12 * * * /usr/local/bin/doas /usr/local/bin/certbot renew --quiet --post-hook \"/usr/local/bin/doas /usr/sbin/service nginx reload\"\n```","title":"Self-signed Certificates - Deploying with Horizon","ref":"deploying-with-horizon.html#self-signed-certificates"},{"type":"extras","doc":"```mermaid\ngraph LR\n A[YOUR APP] --> B[STAGE]\n B --> C[BUILD]\n C --> D[DEPLOY]\n```\n\nThe configuration and install steps described above are run infrequently; usually when versions change or servers are added. \nThe majority of the work in a new deployment is the setup; with that out of the way, you can now build and release your Elixir application into the wild.\n\nThe `stage`, `build`, and `deploy` tasks are the most frequently executed tasks as they are required for each release. Let's look at each step:\n\n- Staging copies the app source to the build machine.\n- Building creates a tarball that is ready to run on a deploy host.\n- Deploy copies the tarball to the build machine and starts the service. (Future: JEDI can allow hot deploys to a running service.)\n\nTo install these scripts for our application, we run:\n```shell\nmix horizon.init\n```\n\nAssuming you have used the default `bin` folder for your project, you should see the following scripts for `my_app1` generated in `bin/`:\n\n```shell\n$ ls -F bin\nbuild-my_app1.sh*         deploy-my_app1.sh*        horizon_helpers.sh\nbuild_script-my_app1.sh*  deploy_script-my_app1.sh  stage-my_app1.sh*\n```","title":"Deploying a Release - Deploying with Horizon","ref":"deploying-with-horizon.html#deploying-a-release"},{"type":"extras","doc":"Before deploying an app we must stage it to the build server and build the app.\nThe `stage` and `build` steps look like:\n\n```shell\n# transfer existing code state to build server\n# (use --force to stage code that is not committed)\n./bin/stage-my_app.sh --force\n\n# build your app on the build server and \n# copy the tarball to .releases/\n./bin/build-my_app.sh\n```\nIn this example, we are using the `--force` option to copy the code to the build server even if it is not committed. \n\nThe `build` step places a tarball in the `.releases` directory. This tarball is ready to be deployed to the production host.","title":"Stage and Build - Deploying with Horizon","ref":"deploying-with-horizon.html#stage-and-build"},{"type":"extras","doc":"The `deploy` step is the final step in the release process.\nIt copies the tarball to the production hosts defined in `releases` in `mix.exs`, extracts the tarball, and (re)starts the service.\n\n```shell\n# deploy the release\n./bin/deploy-my_app.sh\n```\n\nThese steps can be combined into a single command:\n\n```shell\n./bin/stage-my_app.sh --force && ./bin/build-my_app.sh && ./bin/deploy-my_app.sh\n```\n\nIf successful, your application will be running on the production hosts and you can access it via the domain name or IP address of the host.","title":"Deploy - Deploying with Horizon","ref":"deploying-with-horizon.html#deploy"},{"type":"extras","doc":"Here is a summary of the actions taken in each release step.\n\n#### Stage\n- Uses `rsync` or `tar/scp` to copy the current project state to the build host\n#### Build\n- checks if `tailwind` is available and downloads it if needed.\n- installs `mix local.hex`\n- runs `mix deps.get`\n- runs `mix assets.setup.freebsd`\n- runs `mix phx.digest.clean --all`\n- runs `mix assets.deploy`\n- runs `mix release`\n  - Calls `Horizon.Ops.BSD.Step.setup/1` that creates the rcd script\n- stores the tarball in .releases\n- stores the tarball name in `.releases/my_app.data`\n#### Deploy\n\n- adds `my_app_enable=\"YES\"` to `/etc/rc.conf`\n- expands the tarball\n- sets `env.sh` to mode 0400.\n- creates user 'my_app1' (name of your app) if it doesn't exist\n- moves rcd script to `/usr/local/etc/rc.d/my_app1`\n- runs any optional release commands\n- runs `doas service my_app1 restart`\n\nApps are started with the user that has the same name as the app.\nFor example, the release `my_app1` will be run as the user `my_app1`.\nThis allows multiple Elixir/Phoenix apps to reside on the same\nserver, each isolated from the other and individually controlled\nwith the `service` command.","title":"Release Steps Summary - Deploying with Horizon","ref":"deploying-with-horizon.html#release-steps-summary"},{"type":"extras","doc":"Assuming you have the previously described hosts configured and running, here are the steps to deploy `MyApp1` to `demo-web1` and `demo-web2`.\n\n\nWith server\n- mix phx.new my_app1\n- cd my_app1\n- mix ecto.create\n- psql -d my_app1_prod -U 451f8d75-a808-11ef-8e9c-e1aff46a3315 -h demo-pg1\n- Configure `mix.exs`\n- mix horizon.init\n- ./bin/stage-my_app1.sh --force && ./bin/build-my_app1.sh && ./bin/deploy-my_app1.sh\n- configure nginx `http_only`\n- browse to http://demo-web1\n\n> [Horizon provides scripts to backup an existing postgres database and restore it to the new database](horizon-helper-scripts.html#transferring-databases).\n```elixir\n  def project do\n    [\n      app: :my_app1,\n      ...\n      releases: [\n        my_app1: [\n          include_executables_for: [:unix],\n          steps: [&Horizon.Ops.BSD.Step.setup/1, :assemble, :tar],\n          build_host_ssh: \"admin@demo-build\",\n          deploy_hosts_ssh: [\"admin@demo-web1\", \"admin@demo-web2]\n        ],\n      ]\n    ]\n  end\n```","title":"Example Release for `MyApp1` - Deploying with Horizon","ref":"deploying-with-horizon.html#example-release-for-myapp1"},{"type":"extras","doc":"Running `mix release` generates a `script` that is used to start and stop your application. This is a script and not a binary executable. The name of the application binary that gets launched is `beam.smp` with a full path of `/usr/local/my_app/erts- /bin/beam.smp`.\n\nThis is a problem for FreeBSD as it uses the `pid` and the name of the app when searching for a service to stop. For this reason, if you use the default release scripts to `start` and `stop` your app, it will time out when stopping the app.\n\n`Horizon` fixes this by creating a run command file in `/usr/local/etc/rc.d/my_app`. On FreeBSD, you can still use the default scripts:\n\n```\n# Ok to use remote|rpc|eval\n/usr/local/my_app/bin/my_app remote|rpc|eval\n```\n\nfor running **`remote`**, **`rpc`**, or **`eval`**, but for starting/stopping the service, you should use:\n\n```shell\n# service my_app start|stop|restart|status\n```\n\n> If you have an existing `rc_d` script and want to fix it for FreeBSD, simply add `procname=\"*beam.smp\"` to the script. This will allow FreeBSD to find the service and stop it correctly.\n\nThe command usage for the default app is:\n\n```\nUsage: my_app COMMAND [ARGS]\n\nThe known commands are:\n\n  start          Starts the system\n  start_iex      Starts the system with IEx attached\n  daemon         Starts the system as a daemon\n  daemon_iex     Starts the system as a daemon with IEx attached\n  eval \"EXPR\"    Executes the given expression on a new, non-booted system\n  rpc \"EXPR\"     Executes the given expression remotely on the running system\n  remote         Connects to the running system via a remote shell\n  restart        Restarts the running system via a remote command\n  stop           Stops the running system via a remote command\n  pid            Prints the operating system PID of the running system via a remote command\n  version        Prints the release name and version to be booted\n```\n\n\n\n---","title":"Running Phoenix Apps on FreeBSD - Deploying with Horizon","ref":"deploying-with-horizon.html#running-phoenix-apps-on-freebsd"},{"type":"extras","doc":"# Horizon Ops Helper Scripts\n\nHorizon Ops Helper Scripts are a collection of scripts that help with the operational tasks of managing Horizon infrastructure. These scripts are designed to automate common tasks, such as setting up new hosts, configuring databases, and managing backups. By using these scripts, you can save time and reduce the risk of human error in your operations.\n\nBelow is a list of helper scripts available in the Horizon Ops Helper Scripts repository and a brief description of each script's purpose.","title":"Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html"},{"type":"extras","doc":"```\nbsd_install.sh\nUsage: freebsd_setup.sh [user@]host\n```\n\nThe `freebsd_setup.sh` script automates the initial setup of a newly created FreeBSD host. This script automates configuration of elevated privileges, configures SSH keys, disables password login, and updates the system.\n\nIt performs the following tasks:\n- Ensures `doas` command (replacement for `sudo`) is installed.\n- Configures `doas.conf`\n- Verifies SSH `authorized_keys` file is set up and disables password login.\n- Runs `freebsd-update` to update the system.","title":"freebsd_setup.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#freebsd_setup-sh"},{"type":"extras","doc":"This script automates the installation of packages on a remote FreeBSD host. \nIt reads a configuration file and performs the install actions defined therein on the remote host.\n\n```shell\nbsd_install.sh\nUsage: bsd_install.sh [--json] host config_file\n  --json       Output in JSON format (optional)\n  host         [user@]remote_host\n  config_file  Path to the configuration file\n```\n\nSee [Sample Host Configurations](sample-host-configurations.html) for examples of configuration files.","title":"bsd_install.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#bsd_install-sh"},{"type":"extras","doc":"```\nadd_certbot_crontab.sh\nUsage: ops/bin/add_certbot_crontab.sh [user@]host\n```\n\nAdds the cronjob for certbot to renew the SSL certificates and restart nginx.","title":"add_certbot_crontab.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#add_certbot_crontab-sh"},{"type":"extras","doc":"Backup script that runs locally on the postgres backup host.","title":"zfs_snapshot.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#zfs_snapshot-sh"},{"type":"extras","doc":"There are several scripts to help with transferring databases between hosts.\n\n###\tbackup_databases.sh\n```shell\nbackup_databases.sh\n\nUsage: backup_databases.sh [-p port] [-U user] [-o output_dir] host\n\nOptions:\n  -p port         PostgreSQL server port (default: 5432)\n  -U user         PostgreSQL user (default: postgres)\n  -o output_dir   Local directory to store backups (default: current directory)\n```\n\nUses locally installed `psql` to backup all databases on a remote host and transfer them to the local host. Each database is compressed and saved as a separate file in the specified output directory.\n\n###\t`backup_databases_over_ssh.sh`\n```shell\nbackup_databases_over_ssh.sh\nUsage: backup_databases_over_ssh.sh [-o output_dir] [user@]host\n```\n\nIf your database does not have an open port from which to backup, or if you don't have a compatible `psql` client, you can use this script to backup the databases over SSH. This script will connect to the remote host using `ssh` and run `psql` on the remote host to backup the databases.","title":"Transferring Databases - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#transferring-databases"},{"type":"extras","doc":"```shell\nrestore_database.sh\n\nUsage: restore_database.sh [-p port] [-U user] [-d database] backup_file host\n\nOptions:\n  -p port         PostgreSQL server port (default: 5432)\n  -U user         PostgreSQL user (default: postgres)\n  -d database     Database name to restore into\n\nArguments:\n  backup_file     Path to the backup file (required)\n  host            PostgreSQL server host (required)\n```\n\nThis script restores a database from a backup file to a remote PostgreSQL server. It uses the `psql` client to connect to the remote server and restore the database from the specified backup file.","title":"restore_database.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#restore_database-sh"},{"type":"extras","doc":"```shell\nupdate_database_owner.sh\nUsage: update_database_owner.sh [-p port] [-U user] db_name target_user host\n```\n\nThis script updates the owner of a database to a new user. It connects to the specified PostgreSQL server using `psql` at host `HOST` and changes the owner of the specified database to the target user.","title":"update_database_owner.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#update_database_owner-sh"},{"type":"extras","doc":"If you have increased security concerns, Horizon provides scripts to toggle access to the PostgreSQL server from external hosts for individual users.","title":"Control Postgres Access from External Hosts - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#control-postgres-access-from-external-hosts"},{"type":"extras","doc":"```shell\nUsage: turn_off_user_access.sh db_user [user@]remote_host\n```","title":"turn_off_user_access.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#turn_off_user_access-sh"},{"type":"extras","doc":"```shell\nUsage: turn_on_user_access.sh db_user [user@]remote_host\n```","title":"turn_on_user_access.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#turn_on_user_access-sh"},{"type":"extras","doc":"```shell\nturn_off_postgres_access.sh [user@]remote_host\n```","title":"turn_off_postgres_access.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#turn_off_postgres_access-sh"},{"type":"extras","doc":"```shell\nturn_on_postgres_access.sh [user@]remote_host\n```","title":"turn_on_postgres_access.sh - Horizon Ops Helper Scripts","ref":"horizon-helper-scripts.html#turn_on_postgres_access-sh"},{"type":"extras","doc":"# Sample Host Configurations\n\nThis guide contains sample configuration files for setting up a multi-host environment with Horizon.\nUsing these sample files you can build the host collateral that you need to deploy your Elixir/Phoenix application.","title":"Sample Host Configurations","ref":"sample-host-configurations.html"},{"type":"extras","doc":"We start with a sample config file for a build host. \nThe only expectation for this host is to build a release but not run the application.\n\nThis config file uses the current pkg version of erlang `erlang-runtime27` and builds Elixir 1.17.3 from source.\n\nThe `path` is set to the erlang bin directory and must precede the Elixir install.\nThis dynamically sets the path so the Elixir installer knows where to find Erlang.\n\n> **Note:** The `path` is dependent upon the version of erlang that is installed and you will need to update it accordingly.\n\n```build.conf\npkg:ca_root_nss\npkg:gcc\npkg:rsync\npkg:gmake\npkg:git\npkg:erlang-runtime27\n\n# Set the path to erlang so we can install elixir\npath:/usr/local/lib/erlang27/bin\n\nelixir:1.17.3\n```\n\nYou can now configure the `demo-build` build host with:\n\n```shell\n./ops/bin/bsd_install.sh admin@demo-build build.conf\n```","title":"Build Host - Sample Host Configurations","ref":"sample-host-configurations.html#build-host"},{"type":"extras","doc":"A `web` host only needs application specific packages. For example, if your app needs `vips` for use with `vix`, you can install it here.\n\nIn this example, the web host does not need any additional packages.","title":"Web Host - Sample Host Configurations","ref":"sample-host-configurations.html#web-host"},{"type":"extras","doc":"The `reverse-proxy` host is used to route traffic to the web host. It needs to have Nginx installed and configured to route traffic to the web hosts.\n\nWe also install `certbot` on this host to manage SSL certificates.\n\nThe `proxy.conf` file for the `reverse-proxy` host looks like:\n\n```proxy.conf\npkg:nginx\nservice:nginx\npkg:py311-certbot\n```\n\nThe `pkg` commands install the packages and the `service` command starts the Nginx service.\n\nIn this example, we are going to configure the `demo-web1` host as the proxy host:\n\n```\n./ops/bin/bsd_install.sh admin@demo-web1 proxy.conf\n```","title":"Proxy Host - Sample Host Configurations","ref":"sample-host-configurations.html#proxy-host"},{"type":"extras","doc":"The postgres host is your primary database server.\n\nThere are four steps to getting Postgres up and running:\n\n1. Install the version of postgres server and contrib library of your choice\n2. Configure zfs for database snapshots.\n3. Initialize postgres. This configures `postgresql.conf`, `pg_hba.conf` and logging. Logging is at `/var/log/postgresql.log`\n4. Create a database. Creating a database creates a user/password, and updates `pg_hba.conf`. You can also choose the locale and `ctype` of the database.","title":"Postgres Host - Sample Host Configurations","ref":"sample-host-configurations.html#postgres-host"},{"type":"extras","doc":"When creating a database you need to specify how it is to sort strings and how text is encoded. Horizon provides three options:\n\n- `postgres-db-c_mixed_utf8`\n- `postgres-db-c_utf8_full`\n- `postgres-db-us_utf8_full`\n\nThe `c` in `c_mixed_utf8` and `c_utf8_full` stands for `C` locale. This locale sorts strings by byte order. The `us` in `us_utf8_full` stands for `US` locale. This locale sorts strings by dictionary order.\n\n> **If you plan on running physical backups with zfs, install `postgres.zfs-init` BEFORE `postgres.init`.** If `/var/db/postgres/data ` already exists, you will need to create a new zfs dataset manually and move the data directory to the new dataset.\n\nThe `postgres.conf` file for the `postgres` host looks like:\n\n```postgres.con\npkg:postgresql17-server\npkg:postgresql17-contrib\npostgres.zfs-init\npostgres.init\n\n# Encode with UTF8 and sort with byte order\npostgres.db:c_mixed_utf8:mydb\n\n# Other postgresql options\n#postgres.db:us_utf8_full:mydb\n#postgres.db:c_utf8_full:mydb\n```\n\nConfigure your postgres host with:\n\n```\n./ops/bin/bsd_install.sh admin@demo-pg1 postgres.conf\n```","title":"Creating a Database - Sample Host Configurations","ref":"sample-host-configurations.html#creating-a-database"},{"type":"extras","doc":"The postgres backup host is used to store backups of the postgres database.\nFor this host you don't need to initialize postgres, but you do need to install the postgres server, the contrib library and initialize `zfs`.\n\nThe `postgres-backup.conf` file for the `postgres-backup` host looks like:\n\n```postgres-backup.conf\npkg:postgresql17-server\npkg:postgresql17-contrib\npostgres.zfs-init\n```\n\nSetup the postgres backup host with:\n\n```\n./ops/bin/bsd_install.sh admin@demo-pg2 postgres-backup.conf\n```\n\nThat's it! You now have hosts ready for start your Elixir/Phoenix application deployment with Horizon.\n\n---","title":"Postgres Backup Host - Sample Host Configurations","ref":"sample-host-configurations.html#postgres-backup-host"},{"type":"extras","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Sample Proxy Configuration\n\n```elixir\nMix.install([\n  {:horizon, \"~> 0.2\"}\n])\n```","title":"Sample Proxy Configuration","ref":"proxy-conf.html"},{"type":"extras","doc":"```elixir\nuser = \"admin\"\nhost = \"demo-web1\"\n\nprojects = [\n  %Horizon.Project{\n    name: \"my_app1\",\n    server_names: [\"demo-web1\"],\n    http_only: true,\n    # certificate: :letsencrypt,\n    # letsencrypt_domain: \"my_app\",\n    servers: [\n      # Verify PORT is same as in runtime.exs or env.sh.eex\n      %Horizon.Server{internal_ip: \"10.0.0.2\", port: 4000},\n      %Horizon.Server{internal_ip: \"10.0.0.5\", port: 4000}\n    ]\n  }\n]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %Horizon.Project{\n    name: \"my_app1\",\n    server_names: [\"demo-web1\"],\n    certificate: nil,\n    authenticator: nil,\n    cert_path: nil,\n    cert_key_path: nil,\n    letsencrypt_domain: nil,\n    acme_challenge_path: nil,\n    http_only: true,\n    servers: [\n      %Horizon.Server{internal_ip: \"10.0.0.2\", port: 4000},\n      %Horizon.Server{internal_ip: \"10.0.0.5\", port: 4000}\n    ]\n  }\n]\n```\n\n```elixir\n  IO.puts Horizon.NginxConfig.generate(projects)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nload_module /usr/local/libexec/nginx/ngx_mail_module.so;\nload_module /usr/local/libexec/nginx/ngx_stream_module.so;\nworker_processes auto;\n\nevents {\n  worker_connections 1024;\n}\n\nhttp {\n  include mime.types;\n  default_type application/octet-stream;\n  client_max_body_size 6M;\n  sendfile on;\n  keepalive_timeout 65;\n  gzip on;\n  access_log on;\n  access_log /var/log/nginx/access.log;\n\n  map $http_upgrade $connection_upgrade {\n    default upgrade;\n    ''      close;\n  }\n\n  # Define upstream servers (local or remote)\n  upstream backend_my_app1 {\n    ip_hash;\n    server 10.0.0.2:4000;\n    server 10.0.0.5:4000;\n  }\n  \n  # Define server block for HTTP\n  server {\n    listen 80;\n    server_name demo-web1;\n    location / {\n      proxy_pass http://backend_my_app1;\n      proxy_http_version 1.1;\n      proxy_set_header Upgrade $http_upgrade;\n      proxy_set_header Connection $connection_upgrade;\n      proxy_set_header Host $host;\n      proxy_set_header X-Real-IP $remote_addr;\n      proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;\n    }\n  }\n}\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n```elixir\n  Horizon.NginxConfig.send(projects, user, host, action: :restart)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n11:09:45.483 [info] Nginx configuration sent to demo-web1\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok,\n \"Performing sanity check on nginx configuration:\\nStopping nginx.\\nWaiting for PIDS: 75496.\\nPerforming sanity check on nginx configuration:\\nStarting nginx.\\n\"}\n```","title":"Section - Sample Proxy Configuration","ref":"proxy-conf.html#section"},{"type":"extras","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/), and this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.2.2]","title":"Changelog","ref":"changelog.html"},{"type":"extras","doc":"","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Compressed image in docs.","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"## [0.2.1]","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fixed document links in README.md.\n\n\n\n## [0.2.0]","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Added ssl headers to nginx.conf (NginxConfig.generate/4)\n- Added `mix horizon.ops.init` to install HorizonOps scripts.\n- Added .ssh/environment file to store environment variables for ssh connections.\n- Added --verbose option on bsd_install.sh (similar to debug)\n- Added docs for Horizon Ops scripts.\n- Added docs for Proxmox setup.\n- Added docs for FreeBSD Install\n- Added docs for Hetzner Cloud setup\n- Added sample config files for hosts, e.g. [build, web, pg, backup]","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Moved last two arguments of NginxConfig.send/4 to opts\n- Changed log_duration to `off` in postgresql.conf to save space.\n- Changed log_statement to `ddl` in postgresql.conf to save space.\n- Changed `mix horizon.init` to only install project specific deploy scripts.\n- Removed `-u user` option on ops scripts and included optional [user@] for hosts","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fixed bug in NginxConfig.generate to properly handle multiple server names.\n\n\n\n## [0.1.3]","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Added `freebsd_setup.sh` script to help setup first-boot configuration for FreeBSD hosts.\n- Changed host on `bsd_install.sh` to combine user and host into a single argument.\n- Added `postgres.zfs-init` command to `bsd_install.sh` script.","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fixed documentation issues.\n\n\n\n## [0.1.2] - 2024-10-29","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Updates for adding to Hex\n\n## [0.1.1] - 2024-10-29","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Added zfs_snapshot script\n- Added functionality for connecting to VM providers to create host instances (starting with GCE VM hosts).\n- Introduced `Horizon.Target` module utilizing ADTs for deployment targets.\n- Added features to install and configure PostgreSQL on FreeBSD hosts.\n- Implemented initial support for setting up a database follower and moving the master.","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Updated postgres.conf comments\n- Renamed the project from \"Horizon\" to \"HorizonOps\".\n- Updated the structure to include components like `Horizon.GCE`, `Horizon.AWS`, `HorizonBSD`, and `HorizonLinux`.","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Minor bug fixes related to FreeBSD deployment scripts.\n\n## [0.1.0] - 2024-10-19","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Initial release of Horizon.\n- Scripts for deploying Phoenix applications to FreeBSD hosts.\n- Tools for configuring FreeBSD hosts with Elixir/Phoenix releases.\n- Basic support for PostgreSQL installation and configuration on FreeBSD.\n\n[Unreleased]: https://github.com/jfreeze/horizon/compare/v0.1.1...HEAD\n[0.1.1]: https://github.com/jfreeze/horizon/tree/0.1.1","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"BSD 3-Clause License\n\nCopyright (c) 2024, Jim Freeze\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n1. Redistributions of source code must retain the above copyright notice, this\n   list of conditions and the following disclaimer.\n\n2. Redistributions in binary form must reproduce the above copyright notice,\n   this list of conditions and the following disclaimer in the documentation\n   and/or other materials provided with the distribution.\n\n3. Neither the name of the copyright holder nor the names of its\n   contributors may be used to endorse or promote products derived from\n   this software without specific prior written permission.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE\nFOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,\nOR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\nOF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.","title":"LICENSE","ref":"license.html"},{"type":"extras","doc":"# FreeBSD Template Setup\n\nCreating VM templates can save time when deploying new VMs.\n\nThis guide covers the initial setup of a FreeBSD host prior to creating a template from the host.","title":"FreeBSD Template Setup","ref":"freebsd-template-setup.html"},{"type":"extras","doc":"For convenience, we start with defining a host alias in `/etc/hosts` for the existing host.\n\nIn this example our hosts are configured on Hetzner Cloud.\n\nGet the public IP of the server from your provider console.\n![Server Public IP](hetzner-cloud/hetzner-cloud-330.avif)","title":"Server Access Setup - FreeBSD Template Setup","ref":"freebsd-template-setup.html#server-access-setup"},{"type":"extras","doc":"Add the remote server IP to `/etc/hosts`.\nWe know this server will be `web` so we'll add an alias for `demo-web1`.\n\n```\n##\n# Host Database\n#\n# localhost is used to configure the loopback interface\n# when the system is booting.  Do not change this entry.\n##\n127.0.0.1       localhost\n255.255.255.255 broadcasthost\n::1             localhost\n\n5.161.249.144   demo-web1\n```","title":"Add DNS to `/etc/hosts` - FreeBSD Template Setup","ref":"freebsd-template-setup.html#add-dns-to-etc-hosts"},{"type":"extras","doc":"Use `ssh-copy-id` to copy your public key to the remote server.\n\n```shell\nssh-copy-id admin@demo-web1                                                     1 ✘  10:23:52 \n/usr/bin/ssh-copy-id: INFO: Source of key(s) to be installed: \"/Users/jimfreeze/.ssh/id_rsa.pub\"\nThe authenticity of host 'demo-web1 (5.161.249.144)' can't be established.\nED25519 key fingerprint is SHA256:o6XFCLWgF2HQKNlCip4itEqh5S+AY0sYeKEjlrAtDtQ.\nThis key is not known by any other names.\nAre you sure you want to continue connecting (yes/no/[fingerprint])? yes\n/usr/bin/ssh-copy-id: INFO: attempting to log in with the new key(s), to filter out any that are already installed\n/usr/bin/ssh-copy-id: INFO: 1 key(s) remain to be installed -- if you are prompted now it is to install the new keys\n(admin@demo-web1) Password for admin@:\n\nNumber of key(s) added:        1\n\nNow try logging into the machine, with:   \"ssh 'admin@demo-web1'\"\nand check to make sure that only the key(s) you wanted were added.\n```\n\nNow verify you can log in without a password.\n\n```shell\n❯ ssh admin@demo-web1\nFreeBSD 14.1-RELEASE (GENERIC) releng/14.1-n267679-10e31f0946d8\n\nWelcome to FreeBSD!\n\nadmin@:~ $ \n```","title":"Configure passwordless access to the remote server - FreeBSD Template Setup","ref":"freebsd-template-setup.html#configure-passwordless-access-to-the-remote-server"},{"type":"extras","doc":"The `freebsd_setup.sh` script configures new installs of FreeBSD and performs the following tasks:\n- Ensures `doas` command (replacement for `sudo`) is installed.\n- Configures `doas.conf`\n- Verifies SSH `authorized_keys` file is set up and disables password login.\n- Runs `freebsd-update` to update the system.\n\n```sh\nfreebsd_setup.sh admin@demo-web1                                                1 ✘  10:33:44 \n[INFO] Ensuring 'doas' is installed on the remote host...\n[INFO] Running setup script on remote host admin@demo-web1...\n[INFO] Verifying SSH key setup...\n[INFO] Backing up existing configuration files...\n[INFO] Configuring doas...\n[INFO] Configuring .shrc...\n[INFO] Configuring sshd...\n[INFO] Setting SSH-related permissions...\n[INFO] Configuring /boot/loader.conf...\n[INFO] Reloading sshd service...\nPerforming sanity check on sshd configuration.\n[INFO] sshd service reloaded successfully.\n[INFO] Updating the system...\nLooking up update.FreeBSD.org mirrors... 3 mirrors found.\nFetching public key from update1.freebsd.org... done.\nFetching metadata signature for 14.1-RELEASE from update1.freebsd.org... done.\nFetching metadata index... done.\nFetching 2 metadata files... done.\nInspecting system... done.\nPreparing to download files... done.\nFetching 115 patches.....10....20....30....40....50....60....70....80....90....100....110.. done.\nApplying patches... done.\nThe following files will be updated as part of updating to\n14.1-RELEASE-p6:\n/bin/freebsd-version\n/boot/kernel/cfiscsi.ko\n...\nCreating snapshot of existing boot environment... done.\nInstalling updates...\nRestarting sshd after upgrade\nPerforming sanity check on sshd configuration.\nStopping sshd.\nWaiting for PIDS: 84289.\nPerforming sanity check on sshd configuration.\nStarting sshd.\nScanning /usr/share/certs/untrusted for certificates...\nScanning /usr/share/certs/trusted for certificates...\n done.\n[INFO] System updated successfully.\n[INFO] Initial setup complete.\nPlease reboot your system and run the following commands after rebooting:\n  doas pkg upgrade -y\n  doas zfs snapshot zroot/ROOT/default@initial-setup\n[INFO] Remote installs complete. Reboot, upgrade and snapshot remaining.\n  RUN: ssh admin@demo-web1 'doas shutdown -r now'\n```\n\nFollow the instructions to reboot the server and run the upgrade command.\n\n```shell\nssh admin@demo-web1 'doas shutdown -r now'\n\nShutdown NOW!\nshutdown: [pid 83595]\nShutdown NOW!\n\nSystem shutdown time has arrived\n```\n\nLog back in after the server reboots and finish the upgrade.\n```shell\nssh admin@demo-web1\n```\n\n```shell\nadmin@:~ $ doas pkg upgrade -y\nUpdating FreeBSD repository catalogue...\nFetching data.pkg: 100%    7 MiB   7.5MB/s    00:01\nProcessing entries: 100%\nFreeBSD repository update completed. 35521 packages processed.\nAll repositories are up to date.\nChecking for upgrades (1 candidates): 100%\nProcessing candidates (1 candidates): 100%\nChecking integrity... done (0 conflicting)\nYour packages are up to date.\n```\n\n```shell\ndoas zfs snapshot zroot/ROOT/default@initial-setup\n```\n\nYou can verify the snapshot was created with the following command.\n\n```shell\nadmin@:~ $ zfs list -t snapshot\nNAME                                       USED  AVAIL  REFER  MOUNTPOINT\nzroot/ROOT/default@2024-11-19-11:35:22-0   394M      -  2.10G  -\nzroot/ROOT/default@initial-setup           144K      -  2.12G  -\n```\n\nYour server is now set up and ready for Horizon.","title":"Configure the remote server - FreeBSD Template Setup","ref":"freebsd-template-setup.html#configure-the-remote-server"},{"type":"extras","doc":"**We'll use this server as a template to create web hosts, postgres hosts and build hosts.**\n\nIf not using Hetzner Cloud, follow the instructions for your provider to create a snapshot of the server.\n\nFirst, shut down the server.\n```shell\nssh admin@demo-web1 \"doas shutdown -h now\"                         ✔  4h 43m 12s   15:22:12 \nShutdown NOW!\nshutdown: [pid 30486]\nShutdown NOW!\n\nSystem shutdown time has arrived\n```","title":"Creating a Template - FreeBSD Template Setup","ref":"freebsd-template-setup.html#creating-a-template"},{"type":"extras","doc":"Go to the Snapshot tab.\n![Select Snapshot Tab](hetzner-cloud/hetzner-cloud-340.avif)\n\nCreate a snapshot of the server.\n![Create a Snapshot](hetzner-cloud/hetzner-cloud-350.avif)\n\nYou now have a snapshot of the server that can be used to create new servers.\n![Template Snapshot](hetzner-cloud/hetzner-cloud-360.avif)","title":"Create a Snapshot - FreeBSD Template Setup","ref":"freebsd-template-setup.html#create-a-snapshot"},{"type":"extras","doc":"# Hetzner Cloud Setup Guide\n\nA quick guide to creating a FreeBSD host template on [Hetzner Cloud](https://console.hetzner.cloud/projects)\n\nThis guide will walk you through setting up a new Hetzner Cloud project, creating a firewall, a network, and adding servers.\n\nAfter creating a project, our strategy is to create network and firewall collateral first. \nThen we will add servers to the network and create a new server and save it as a template.\n\nThat template will be used to create new servers in the future.","title":"Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html"},{"type":"extras","doc":"- [Create a new project](#create-a-new-project)\n- [Create a Firewall](#create-a-firewall)\n  - [Name SSH and Ping Firewall Rules](#ssh-and-ping-firewall-rules)\n  - [SSH and Ping Firewall Rules](#ssh-and-ping-firewall-rules)\n  - [Web Server Rules](#web-server-rules)\n  - [Phoenix Dev Rule and LAN Rules](#phoenix-dev-rule-and-lan-rules)\n  - [Name the Firewall](#name-the-firewall)\n- [Create a Network](#create-a-network)","title":"FreeBSD Host Installation on Hetzner Cloud Summary - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#freebsd-host-installation-on-hetzner-cloud-summary"},{"type":"extras","doc":"Browse to [Hetzner Cloud](https://console.hetzner.cloud/projects) and click \"New project\".\n\n![Create a project](hetzner-cloud/hetzner-cloud-020.avif)\n\nand add your project.\n\n![Create a project](hetzner-cloud/hetzner-cloud-030.avif)","title":"Create a new project - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#create-a-new-project"},{"type":"extras","doc":"In the project page, you can configure networks, firewalls, and servers.\n\n![Project page](hetzner-cloud/hetzner-cloud-050.avif)","title":"Project Overview - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#project-overview"},{"type":"extras","doc":"> **If you are using a Cloudflare Argo Tunnel or other service to expose your app to the internet, the following firewall rules may need to be adapted.**\n\nClick \"Create Firewall\" to create a new firewall.\n![Create a Firewall](hetzner-cloud/hetzner-cloud-060.avif)","title":"Create a Firewall - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#create-a-firewall"},{"type":"extras","doc":"Here we will create `inbound` rules and internal LAN rules.\n\nHetzner provides rules for `ssh` and `ping` but does not name them.\nFeel free to add appropriate names to the rules.\n\n![SSH and Ping Rules](hetzner-cloud/hetzner-cloud-070.avif)","title":"SSH and Ping Firewall Rules - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#ssh-and-ping-firewall-rules"},{"type":"extras","doc":"Next, add rules for HTTP and HTTPS.\n![Web Server Rules](hetzner-cloud/hetzner-cloud-080.avif)\n\n###  Phoenix Dev Rule and LAN Rules\nOpen port `4000` for the Phoenix server. \nYou can close this port in your production firewall.\nOpening this port is helpful when testing deployment because \nthis can be accessed when `nginx` is running on port 80 and 443.\n\nAlso shown in the image below are the LAN rules that allow servers to talk to each other.\nThe web hosts will need to access the postgres server on port 5432 and\nthe postgres backup server will need to access the postgres server for backups.\n\nWe open all the ports for the LAN because the servers are in a private network.\n![Port 4000 and LAN](hetzner-cloud/hetzner-cloud-100.avif)","title":"Web Server Rules - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#web-server-rules"},{"type":"extras","doc":"![Name the Firewall](hetzner-cloud/hetzner-cloud-110.avif)","title":"Name the Firewall - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#name-the-firewall"},{"type":"extras","doc":"Move next to the network tab and create a new network.\n![Create a Network](hetzner-cloud/hetzner-cloud-120.avif)","title":"Create a Network - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#create-a-network"},{"type":"extras","doc":"Select the network zone that matches where your servers are located.\nThe default IP range is usually sufficient.\n![Network Zone](hetzner-cloud/hetzner-cloud-130.avif)","title":"Network Zone - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#network-zone"},{"type":"extras","doc":"Move to the servers tab to add a server to the network.\nClick \"Add Server\" to create a new server.\n![Add Servers](hetzner-cloud/hetzner-cloud-140.avif)","title":"Add Servers - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#add-servers"},{"type":"extras","doc":"Hetzner does not provide a FreeBSD image, so we will select an existing image and install \nFreeBSD on it using one of the FreeBSD ISO images hosted by Hetzner.\n\nWhich image you select is not important because we will be installing FreeBSD from an ISO image.\n![Select Server Image](hetzner-cloud/hetzner-cloud-170.avif)","title":"Select an Image - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#select-an-image"},{"type":"extras","doc":"Select if you want to use a shared or dedicated CPU.\n![Select Server Type](hetzner-cloud/hetzner-cloud-180.avif)\n\nSelect the number of CPUs and memory.\n![Select Server CPU and Memory](hetzner-cloud/hetzner-cloud-190.avif)","title":"Select Server Type - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#select-server-type"},{"type":"extras","doc":"![Select Server Firewall](hetzner-cloud/hetzner-cloud-160.avif)","title":"Select the Firewall - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#select-the-firewall"},{"type":"extras","doc":"![Select Server Network](hetzner-cloud/hetzner-cloud-200.avif)","title":"Select the Network - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#select-the-network"},{"type":"extras","doc":"![Name the Server](hetzner-cloud/hetzner-cloud-210.avif)","title":"Name the Server - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#name-the-server"},{"type":"extras","doc":"Go to the server dashboard page and click \"ISO Images\" to add the FreeBSD ISO image.\n![ISO Images](hetzner-cloud/hetzner-cloud-230.avif)","title":"Server Dashboard Page - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#server-dashboard-page"},{"type":"extras","doc":"Type `freebsd` in the search box to find the FreeBSD ISO images and mount the most recent image.\n\n![Mount FreeBSD ISO](hetzner-cloud/hetzner-cloud-240.avif)","title":"Mount FreeBSD ISO - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#mount-freebsd-iso"},{"type":"extras","doc":"Click the console icon to open a console.\n![Open a Console](hetzner-cloud/hetzner-cloud-250.avif)","title":"Launch Console - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#launch-console"},{"type":"extras","doc":"Once in the console, click \"Ctl + Alt + Del\" to reboot the server.\n**Note that you will need to click into the console and enter keystrokes for the console to recognize and mount the ISO.** (Don't ask me why.)\n\n![Hetzner Console](hetzner-cloud/hetzner-cloud-260.avif)","title":"Open a Console - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#open-a-console"},{"type":"extras","doc":"When the ISO is recognized and booted, you will see the FreeBSD boot installer. Press enter to boot the installer.\n\nFollow the steps for [FreeBSD install](freebsd-install.html). \nThe notable difference is that you will need to select the `vtnet0` network interface and configure the IPv6 address.\nWe will configure `vtnet1` for the LAN interface in a later step.\n![FreeBSD Boot Installer](hetzner-cloud/hetzner-cloud-270.avif)\n\nThe network and IPv6 configuration shown below are additional steps not found in the [FreeBSD install](freebsd-install.html) guide.","title":"FreeBSD Boot Installer - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#freebsd-boot-installer"},{"type":"extras","doc":"Choose `vtnet0` for the public network interface. `vtne1` will be used for the private LAN interface.\n![vtnet0 Configuration](hetzner-cloud/hetzner-cloud-280.avif)","title":"vtnet0 Configuration - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#vtnet0-configuration"},{"type":"extras","doc":"Configure the IPv6 address for the public network interface.\n![IPv6 Configuration](hetzner-cloud/hetzner-cloud-290.avif)","title":"IPv6 Configuration - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#ipv6-configuration"},{"type":"extras","doc":"![SLAAC Autoconfiguration](hetzner-cloud/hetzner-cloud-300.avif)\n\nContinue with the [FreeBSD install](freebsd-install.html) guide.","title":"SLAAC Autoconfiguration for IPv6 - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#slaac-autoconfiguration-for-ipv6"},{"type":"extras","doc":"After the installation is complete, you will need to reboot the server and then **unmount the ISO image** before it starts up.\n![Reboot](hetzner-cloud/hetzner-cloud-310.avif)","title":"Unmount ISO - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#unmount-iso"},{"type":"extras","doc":"Reboot the server and unmount the ISO. Be quick! :-)\n![Unmount ISO](hetzner-cloud/hetzner-cloud-320.avif)","title":"Reboot - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#reboot"},{"type":"extras","doc":"You now have a FreeBSD server that is almost ready to be used as a template for future servers.\nFollow the steps in [FreeBSD Template Setup](freebsd-template-setup.html) to complete the template setup.","title":"Template Creation - Hetzner Cloud Setup Guide","ref":"hetzner-cloud.html#template-creation"},{"type":"extras","doc":"# Hetzner Cloud Host Instantiation\n\nThis guide covers creating multiple hosts from a template on Hetzner Cloud to create\na horizontally scalable infrastructure.","title":"Hetzner Cloud Host Instantiation","ref":"hetzner-cloud-host-instantiation.html"},{"type":"extras","doc":"In the Image section, click on the `Snapshots` tab and select the snapshot you created in the previous guide.\n\n![Select Snapshot](hetzner-template-install/hetzner-cloud-template-010.avif)","title":"Select a host Image - Hetzner Cloud Host Instantiation","ref":"hetzner-cloud-host-instantiation.html#select-a-host-image"},{"type":"extras","doc":"For the scalable infrastructure, we'll create multiple hosts from the snapshot.\nAdd four servers to the list and name them `web2`, `pg1`, `pg2`, and `build`.\n![Name the Servers](hetzner-template-install/hetzner-cloud-template-050.avif)","title":"Name the Hosts - Hetzner Cloud Host Instantiation","ref":"hetzner-cloud-host-instantiation.html#name-the-hosts"},{"type":"extras","doc":"Create the hosts and note the IP addresses, both public and private.\n![Server Summary](hetzner-template-install/hetzner-cloud-template-060.avif)","title":"Create the Hosts - Hetzner Cloud Host Instantiation","ref":"hetzner-cloud-host-instantiation.html#create-the-hosts"},{"type":"extras","doc":"# FreeBSD Installation\n\nInstalling FreeBSD from an ISO image is a simple process that takes around 3-5 minutes.\nIf you're used to using containers, you can think of this step as the equivalent of searching for and creating a new container. The upside is you know exactly what is going into your FreeBSD installation.\n\nFor most systems. this process only needs to be performed occasionally as FreeBSD systems are updated. This example shows the process for a new installation of FreeBSD 14.1 RELEASE.","title":"FreeBSD Installation","ref":"freebsd-install.html"},{"type":"extras","doc":"Select the Boot Installer by hitting enter.\n![Start Installer](freebsd-install/freebsd-install-010.avif)","title":"Start the Installer - FreeBSD Installation","ref":"freebsd-install.html#start-the-installer"},{"type":"extras","doc":"![Select Install](freebsd-install/freebsd-install-020.avif)","title":"Select Install - FreeBSD Installation","ref":"freebsd-install.html#select-install"},{"type":"extras","doc":"![Select Your Keyboard](freebsd-install/freebsd-install-030.avif)","title":"Select Your Keyboard - FreeBSD Installation","ref":"freebsd-install.html#select-your-keyboard"},{"type":"extras","doc":"You can leave the hostname blank for now as we will set it later during the initial setup.\n\n![Hostname](freebsd-install/freebsd-install-040.avif)","title":"Hostname - FreeBSD Installation","ref":"freebsd-install.html#hostname"},{"type":"extras","doc":"We add the ports tree to save time in case it is needed later and we add the source tree for convenience.\n![System Components](freebsd-install/freebsd-install-050.avif)","title":"System Components - FreeBSD Installation","ref":"freebsd-install.html#system-components"},{"type":"extras","doc":"We will use the entire disk for the installation. If you have a specific partitioning scheme in mind, you can select the manual option.\n\n![Disk Setup](freebsd-install/freebsd-install-060.avif)","title":"Disk Setup - FreeBSD Installation","ref":"freebsd-install.html#disk-setup"},{"type":"extras","doc":"Use the defaults for ZFS configuration.\n![ZFS Configuration](freebsd-install/freebsd-install-070.avif)","title":"ZFS Configuration - FreeBSD Installation","ref":"freebsd-install.html#zfs-configuration"},{"type":"extras","doc":"This will depend o your hosting setup, but usually we offload any disk redundancy to the hosting provider.\n\nIn a typical Horizon setup, the two host types (web and postgres) do not need redundancy as the web hosts are stateless and can be recreated easily and the postgres host is continually backed up (and possibly mirrored).\n\n\n![Device Type](freebsd-install/freebsd-install-080.avif)","title":"Device Type - FreeBSD Installation","ref":"freebsd-install.html#device-type"},{"type":"extras","doc":"Select the device to format.\n\n![Drive Selection](freebsd-install/freebsd-install-090.avif)","title":"Drive Selection - FreeBSD Installation","ref":"freebsd-install.html#drive-selection"},{"type":"extras","doc":"![Confirm Formatting](freebsd-install/freebsd-install-100.avif)","title":"Confirm Formatting - FreeBSD Installation","ref":"freebsd-install.html#confirm-formatting"},{"type":"extras","doc":"![Checksum Verification](freebsd-install/freebsd-install-110.avif)","title":"Checksum Verification - FreeBSD Installation","ref":"freebsd-install.html#checksum-verification"},{"type":"extras","doc":"![Installation (Archive Extraction)](freebsd-install/freebsd-install-120.avif)","title":"Installation (Archive Extraction) - FreeBSD Installation","ref":"freebsd-install.html#installation-archive-extraction"},{"type":"extras","doc":"![Set Root Password](freebsd-install/freebsd-install-130.avif)","title":"Set Root Password - FreeBSD Installation","ref":"freebsd-install.html#set-root-password"},{"type":"extras","doc":"![Network Configuration](freebsd-install/freebsd-install-140.avif)","title":"Network Configuration - FreeBSD Installation","ref":"freebsd-install.html#network-configuration"},{"type":"extras","doc":"![Add IPv4 Interface](freebsd-install/freebsd-install-150.avif)","title":"Add IPv4 Interface - FreeBSD Installation","ref":"freebsd-install.html#add-ipv4-interface"},{"type":"extras","doc":"![Use DHCP](freebsd-install/freebsd-install-160.avif)","title":"Use DHCP - FreeBSD Installation","ref":"freebsd-install.html#use-dhcp"},{"type":"extras","doc":"This can skipped if you don't plan to use IPv6.\n![IPv6 Configuration](freebsd-install/freebsd-install-170.avif)","title":"IPv6 Configuration - FreeBSD Installation","ref":"freebsd-install.html#ipv6-configuration"},{"type":"extras","doc":"![Confirm Network Configuration](freebsd-install/freebsd-install-180.avif)","title":"Confirm Network Configuration - FreeBSD Installation","ref":"freebsd-install.html#confirm-network-configuration"},{"type":"extras","doc":"![Select Timezone](freebsd-install/freebsd-install-200.avif)","title":"Select Timezone - FreeBSD Installation","ref":"freebsd-install.html#select-timezone"},{"type":"extras","doc":"![Select Country](freebsd-install/freebsd-install-210.avif)","title":"Select Country - FreeBSD Installation","ref":"freebsd-install.html#select-country"},{"type":"extras","doc":"This is the city closest to your server. If you don't see your city, select the closest one.\n![Select Locality](freebsd-install/freebsd-install-220.avif)","title":"Select Locality - FreeBSD Installation","ref":"freebsd-install.html#select-locality"},{"type":"extras","doc":"![Confirm Timezone](freebsd-install/freebsd-install-230.avif)]","title":"Confirm Timezone - FreeBSD Installation","ref":"freebsd-install.html#confirm-timezone"},{"type":"extras","doc":"![Skip Date Setup](freebsd-install/freebsd-install-240.avif)","title":"Skip Set Date - FreeBSD Installation","ref":"freebsd-install.html#skip-set-date"},{"type":"extras","doc":"![Skip Time Setup](freebsd-install/freebsd-install-250.avif)","title":"Skip Set Time - FreeBSD Installation","ref":"freebsd-install.html#skip-set-time"},{"type":"extras","doc":"It's a good idea to add `ntpd` and `ntpd_sync_on_start` as VM clocks can drift significantly.\nAdding `local_unbound` can speed up DNS lookups but is optional.\n\n![System Configuration](freebsd-install/freebsd-install-260.avif)","title":"System Configuration - FreeBSD Installation","ref":"freebsd-install.html#system-configuration"},{"type":"extras","doc":"Select all the hardening options.\n\n![System Hardening](freebsd-install/freebsd-install-270.avif)","title":"System Hardening - FreeBSD Installation","ref":"freebsd-install.html#system-hardening"},{"type":"extras","doc":"![Add a User](freebsd-install/freebsd-install-280.avif)","title":"Add a User - FreeBSD Installation","ref":"freebsd-install.html#add-a-user"},{"type":"extras","doc":"This example adds the `admin` user and sets their group to `wheel`.\nAdding the user to the `wheel` group is required to give your user elevated privileges if using the `freebsd_setup.sh` script.\n\n![Add User Console](freebsd-install/freebsd-install-290.avif)","title":"Add User Console - FreeBSD Installation","ref":"freebsd-install.html#add-user-console"},{"type":"extras","doc":"![Exit Guided Install](freebsd-install/freebsd-install-320.avif)","title":"Exit Guided Install - FreeBSD Installation","ref":"freebsd-install.html#exit-guided-install"},{"type":"extras","doc":"![Manual Configuration](freebsd-install/freebsd-install-330.avif)","title":"Manual Configuration - FreeBSD Installation","ref":"freebsd-install.html#manual-configuration"},{"type":"extras","doc":"At the end of installation but before rebooting, add the `doas` package to your system \nand configure privileges for the `wheel` group.\nThe simple `doas` configuration will allow you to run commands to setup your hosts.\nWhen you run the `freebsd_setup.sh` script it will enhance `doas.conf`.\n\n![Add doas oops](freebsd-install/freebsd-install-340.avif)","title":"Add `doas.conf` - FreeBSD Installation","ref":"freebsd-install.html#add-doas-conf"},{"type":"extras","doc":"![Add doas](freebsd-install/freebsd-install-350.avif)","title":"Oops. Create the directory first. - FreeBSD Installation","ref":"freebsd-install.html#oops-create-the-directory-first"},{"type":"extras","doc":"![Add the pkg doas](freebsd-install/freebsd-install-360.avif)","title":"Add the pkg `doas` - FreeBSD Installation","ref":"freebsd-install.html#add-the-pkg-doas"},{"type":"extras","doc":"![Exit](freebsd-install/freebsd-install-370.avif)","title":"Exit - FreeBSD Installation","ref":"freebsd-install.html#exit"},{"type":"extras","doc":"![Reboot](freebsd-install/freebsd-install-380.avif)\n\nYour FreeBSD system is now installed and ready for the next steps.\n\n---","title":"Reboot - FreeBSD Installation","ref":"freebsd-install.html#reboot"},{"type":"extras","doc":"```conf\n#\n# Horizon.Ops - Sample Config for a build host\n#\n\n# The `pkg` option installs the specified package using the package manager.\npkg:ca_root_nss\npkg:gcc\npkg:rsync\npkg:gmake\npkg:git\npkg:erlang-runtime27\n\n# The `path` option sets a runtime path for the script\n# Set the path to erlang so we can install elixir\npath:/usr/local/lib/erlang27/bin\n\n# The `elixir` option pulls the specified version of Elixir from github and builds Elixir from source\nelixir:1.17.3\n```","title":"build_conf","ref":"build_conf.html"},{"type":"extras","doc":"```conf\n#\n# Horizon.Ops - Sample Config for a reverse-proxy and web server\n#\n\npkg:nginx\n\n# Configure and start the nginx service\nservice:nginx\npkg:py311-certbot\n```","title":"web_proxy_conf","ref":"web_proxy_conf.html"},{"type":"extras","doc":"```conf\n#\n# Horizon.Ops - Sample Config for a Postgres Server\n#\n# This installation script sets up a PostgreSQL server,\n# initializes the database, and creates multiple databases\n# with different locale and encoding settings.\n#\n# Install Options\n# The script uses several configuration options to set up and customize the PostgreSQL server.\n#\n# 1. Install Packages\n#\n# Option: pkg\n#\n#     pkg: \n#\n# The `pkg` option installs the specified package using the package manager.\n#\n# Examples:\n#\n#     pkg:postgresql16-server\n#     pkg:postgresql16-contrib\n#\n# These commands install the PostgreSQL 17 server and additional contributed utilities.\n# Note that you don't need to specify the full name of the package.\n#\n#\n# 2. Initialize ZFS\n#\n# Option: postgres.zfs-init\n#\n# This option initializes ZFS for the PostgreSQL server.\n# It creates a ZFS pool and sets up the mount point for the database.\n# This must be run before initializing the PostgreSQL server.\n#\n# Example:\n#\n#    postgres.zfs-init\n#\n#\n# 3. Initialize PostgreSQL\n#\n# Option: postgres.init\n#\n# This option initializes the PostgreSQL server.\n# It calls `initdb` starts the database server.\n#\n# Example:\n#\n#     postgres.init\n#\n#\n# 4. Create Databases with Specific Locale Settings\n#\n# Option: postgres.db: : \n#\n# This option creates a PostgreSQL database with a specific encoding type and locale settings.\n# The   specifies the collation and character type (ctype) settings,\n# and   is the name of the database to be created.\n#\n# Encoding Types:\n#\n# c_mixed_utf8: Collate = C,           CType = C.UTF-8 (default, uses template1)\n# c_utf8_full:  Collate = C.UTF-8,     CType = C.UTF-8\n# us_utf8_full: Collate = en_US.UTF-8, CType = en_US.UTF-8\n#\n# Examples:\n#\n#     postgres.db:c_mixed_utf8:mydb1\n#     postgres.db:c_utf8_full:mydb2\n#     postgres.db:us_utf8_full:mydb3\n#\n# These commands create three databases:\n# mydb1 with collation C and ctype C.UTF-8\n# mydb2 with both collation and ctype set to C.UTF-8\n# mydb3 with both collation and ctype set to en_US.UTF-8\n#\n\npkg:postgresql17-server\npkg:postgresql17-contrib\npostgres.zfs-init\npostgres.init\n\n# Create individual database\npostgres.db:c_mixed_utf8:my_app\n```","title":"postgres_conf","ref":"postgres_conf.html"},{"type":"extras","doc":"```conf\n#\n# Horizon.Ops - Sample Config for a Postgres Backup Server\n#\n# Install postgres for convenience if we need to restore from backup\n# or stand up the backup as the replacement server\n#\n\npkg:postgresql17-server\npkg:postgresql17-contrib\n\n# Initialize ZFS for the PostgreSQL Backup server\npostgres.zfs-init\n\n```","title":"postgres_backup_conf","ref":"postgres_backup_conf.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}